﻿using OrBit.MESInterface.Application.Contracts.Dtos;
using OrBit.MESInterface.Domain.InterfaceConfiguration.Entities;
using OrBit.MESInterface.Infrastructure.Common;
using System;
using System.Threading;
using System.Threading.Tasks;
using Volo.Abp.Application.Services;

namespace OrBit.MESInterface.Application.Contracts.Interfaces
{
    public interface IMESInterfaceConfigService : IApplicationService
    {
        /// <summary>
        /// 获取完整的接口配置信息
        /// </summary>
        /// <param name="InterfaceType">接口类型</param>
        /// <param name="cancellationToken">取消状态</param>
        /// <returns></returns>
        Task<MESInterfaceConfigDto> FindInterfaceConfigDTO(string InterfaceType, string InterfaceServiceName, CancellationToken cancellationToken);

        /// <summary>
        /// 获取完整的接口配置信息
        /// </summary>
        /// <param name="Interface_Request_Config">接口配置对象</param>
        /// <param name="InterfaceServiceName">服务类型</param>
        /// <param name="cancellationToken">取消令牌</param>
        /// <returns></returns>
        MESInterfaceConfigDto FindInterfaceConfigDTO(Interface_Request_Configuration Interface_Request_Config, string InterfaceServiceName, CancellationToken cancellationToken = default);

        /// <summary>
        /// 获取接口配置对象
        /// </summary>
        /// <param name="InterfaceType"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        Task<Interface_Request_Configuration> FindInterfaceConfig(string InterfaceType, CancellationToken cancellationToken);

        /// <summary>
        /// 指定请求参数获取接口服务请求数据
        /// </summary>
        /// <param name="InterfaceConfig">接口配置信息</param>
        /// <param name="RequestURL">自定义的请求地址</param>
        /// <param name="InterfaceServiceName">接口服务名称</param>
        /// <param name="RequestDataParameters">指定参数集合</param>
        /// <param name="cancellationToken">取消状态</param>
        /// <returns></returns>
        Task<RequestData> BuildRequestDataByInterfaceTypeAsync(MESInterfaceConfigDto InterfaceConfig, string RequestURL, DateTime QueryDateEnd, string RequestDataParameters, CancellationToken cancellationToken, string EventID);
    }
}