using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime;
using System.Threading.Tasks;
using OrBit.MESInterface.Application.Contracts.AppUpdata;
using OrBit.MESInterface.Application.Contracts.AppUpdata.Dto;
using OrBit.MESInterface.Infrastructure;
using Volo.Abp.Application.Services;

namespace OrBit.MESInterface.Application.Service.AppUpdata;

public class AppUpdataServer:ApplicationService, IAppUpdataServer
{
    public Task<ExecutionResult> GetDriveInfo()
    {
        DriveInfo[] allDirves = DriveInfo.GetDrives();
        List<DriveInfoDto> dirInfoList = new List<DriveInfoDto>();
        foreach (DriveInfo item in allDirves)
        {
            DriveInfoDto dirInfoDto = new DriveInfoDto
            {
                DriveName = item.Name,
               
                IsReady = item.IsReady,
            };
            
            if(item.IsReady)
            {
                dirInfoDto.DriveFormat = item.DriveFormat.ToString();
                dirInfoDto.RootDirectory = item.RootDirectory.FullName;
                dirInfoDto.TotalSize = item.TotalSize;
                dirInfoDto.TotalFreeSpace = item.TotalFreeSpace;
                dirInfoDto.VolumeLabel = item.VolumeLabel;
                dirInfoDto.DriveType = item.DriveType.ToString();
            }
          
            dirInfoList.Add(dirInfoDto);
        }

        return Task.FromResult(ExecutionResult.Success("获取驱动器成功",dirInfoList));
    }

    public  Task<ExecutionResult> GetAllDirectoryAndFileInfoAsync(string rootPath)
    {
       
     var dir=  GetFileFullPath( null, rootPath);
        return Task.FromResult(ExecutionResult.Success("获取成功",dir));
    }

    public Task<ExecutionResult> GetDirectoryAndFileInfoAsync(string rootPath)
    {
        if(!Directory.Exists(rootPath))
        {
            return Task.FromResult(ExecutionResult.Failed("目录不存在"));
        }

        DirectoryInfo dir = new DirectoryInfo(rootPath);
        DirectoryInfoDto directoryInfoDto = new DirectoryInfoDto
        {
            FullPath = dir.FullName,
            DirectoryName = dir.Name
        };
        FileSystemInfo[] fsinfos = null;
        try
        {
            //检索表示当前目录的文件和子目录
            fsinfos = dir.GetFileSystemInfos();
        }
        catch (Exception)
        {
            fsinfos = new FileSystemInfo[0];
        }

        //遍历检索的文件和子目录
        foreach (FileSystemInfo fsinfo in fsinfos)
        {
            //判断是否为空文件夹　　
            if (fsinfo is DirectoryInfo)
            {
                var cdir = new DirectoryInfoDto
                {
                    FullPath = fsinfo.FullName,
                    DirectoryName = fsinfo.Name
                };
                directoryInfoDto.SubDirectories.Add(cdir);
               
            }
            else//文件
            {
                Console.WriteLine(fsinfo.FullName);
                //  var belongingDevice = Path.GetDirectoryName(fsinfo.FullName);
                // belongingDevice = belongingDevice.Replace(belongingDevice.Substring(0, belongingDevice.LastIndexOf("\\") + 1), "");
                directoryInfoDto.Files.Add(new FileInfoDto()
                {
                    FileName = Path.GetFileNameWithoutExtension(fsinfo.FullName),
                    FullPath = fsinfo.FullName,
                    FileNameEx = fsinfo.Name,
                });
            }

        }

        return Task.FromResult(ExecutionResult.Success("查询成功",directoryInfoDto));
    }
    
    /// <summary>
    /// 递归获取指定目录下所有的目录与目录下的文件
    /// </summary>
    /// <param name="directoryInfo">目录与文件的集合</param>
    /// <param name="dirInfo">上层目录信息</param>
    /// <param name="dirs">目录</param>
    private DirectoryInfoDto GetFileFullPath( DirectoryInfoDto dirInfo,string dirs)
    {
        //绑定到指定的文件夹目录
        DirectoryInfo dir = new DirectoryInfo(dirs);
        var InDir= new DirectoryInfoDto
        {
            FullPath = dir.FullName,
            DirectoryName = dir.Name
        };
        //第一次调用
        if (dirInfo == null)
        {
            dirInfo = InDir;

        }


        FileSystemInfo[] fsinfos = null;
        try
        {
            //检索表示当前目录的文件和子目录
            fsinfos = dir.GetFileSystemInfos();
        }
        catch (Exception)
        {
            fsinfos = new FileSystemInfo[0];
        }
        

        //遍历检索的文件和子目录
        foreach (FileSystemInfo fsinfo in fsinfos)
        {
            //判断是否为空文件夹　　
            if (fsinfo is DirectoryInfo)
            {
                var cdir = new DirectoryInfoDto
                {
                    FullPath = fsinfo.FullName,
                    DirectoryName = fsinfo.Name
                };
                dirInfo.SubDirectories.Add(cdir);
                //递归调用
                GetFileFullPath(cdir,fsinfo.FullName);
            }
            else//文件
            {
                Console.WriteLine(fsinfo.FullName);
              //  var belongingDevice = Path.GetDirectoryName(fsinfo.FullName);
               // belongingDevice = belongingDevice.Replace(belongingDevice.Substring(0, belongingDevice.LastIndexOf("\\") + 1), "");
               dirInfo.Files.Add(new FileInfoDto()
               {
                   FileName = Path.GetFileNameWithoutExtension(fsinfo.FullName) ,
                   FullPath = fsinfo.FullName,
                   FileNameEx= fsinfo.Name,
                    
               });
            }
            
        }

        return dirInfo;
        //将得到的文件全路径放入到集合中
        //return Task.CompletedTask;  
    }
}