﻿using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using OrBit.MESInterface.Application.Contracts.Dtos;
using OrBit.MESInterface.Application.Contracts.Interfaces;
using OrBit.MESInterface.Domain.InterfaceConfiguration.Entities;
using OrBit.MESInterface.Domain.InterfaceConfiguration.Repository;
using OrBit.MESInterface.Infrastructure;
using OrBit.MESInterface.Infrastructure.Common;
using OrBit.MESInterface.Infrastructure.CreateTable;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using Volo.Abp;
using Volo.Abp.Application.Services;
using Volo.Abp.Domain.Repositories;

namespace OrBit.MESInterface.Application.Service
{
    public class DownloadDataAppService : ApplicationService, IDownloadDataAppService
    {
        private readonly ILogger<DownloadDataAppService> _logger;

        private readonly IServiceProvider _serviceProvider;

        private readonly DefaultHttpClientHelper _httpClient;

        private readonly IRepository<Interface_ResultDataConfigInfo> _ResultDataConfigInfos;
        private readonly IInterface_Request_ConfigurationRepository _interface_Request_ConfigRepository;
        private readonly IRepository<Interface_Post_ServiceCode> _interface_Post_ServicesRepository;

        private readonly IMESInterfaceConfigService _interfaceConfigService;

        public DownloadDataAppService(IServiceProvider serviceProvider, ILogger<DownloadDataAppService> logger, DefaultHttpClientHelper defaultHttpClient,
            IRepository<Interface_ResultDataConfigInfo> _ResultDataConfigInfos,
            IInterface_Request_ConfigurationRepository _interface_Request_ConfigRepository,
            IRepository<Interface_Post_ServiceCode> _interface_Post_ServicesRepository,
            IMESInterfaceConfigService _interfaceConfigService

         )
        {
            _logger = logger;
            _httpClient = defaultHttpClient;
            _serviceProvider = serviceProvider;
            this._ResultDataConfigInfos = _ResultDataConfigInfos;
            this._interface_Request_ConfigRepository = _interface_Request_ConfigRepository;
            this._interfaceConfigService = _interfaceConfigService;
            this._interface_Post_ServicesRepository = _interface_Post_ServicesRepository;
        }

        //public async Task<string> AllInterfaceDownload(IMESInterfaceConfigService _interfaceConfigService, string InterfaceType)
        //{
        //    var InterfaceConfig = await _interfaceConfigService.FindInterfaceConfig(InterfaceType, default);
        //    DateTime QueryDateEnd = DateTime.Now;
        //    List<Task> tasks = new List<Task>();
        //    var Post_ServiceCodes = InterfaceConfig._Post_ServiceCodes.ToList();
        //    if (InterfaceConfig.RequestMethod.ToUpper() == "POST")
        //    {
        //        Post_ServiceCodes.ForEach(async Service =>
        //       {
        //           if (Service.IsAutoGetOrPutData)
        //           {
        //               var EventID = Guid.NewGuid().ToString();
        //               var a = await SingleInterfaceDownload(InterfaceConfig, Service.InterfaceServiceName, EventID);
        //               //  tasks.Add( SingleInterfaceDownload(InterfaceConfig, Service.InterfaceServiceName));
        //           }
        //       });
        //    }
        //    return $"接口自动同步数据处理完成。具体结果请参考日志";
        //    //if (tasks.Count > 0)
        //    //{
        //    //    Task.WaitAll(tasks.ToArray());
        //    //    return $"{tasks.Count}个接口自动同步数据处理完成。具体结果请参考日志";
        //    //}
        //    //else
        //    //{
        //    //    return $"{tasks.Count}个接口需要自动同步数据";
        //    //}
        //}

        //public async Task<ExecutionResult> SingleInterfaceDownload(Interface_Request_Configuration InterfaceType, string InterfaceServiceName, string EventID, string RequestURL="", string RequestDataParameters = "")
        //{
        //    using (var scope = _serviceProvider.CreateScope())
        //    {
        //        var MESInterfaceConfigService = _serviceProvider.GetService<IMESInterfaceConfigService>();
        //        var __interface_Post_ServicesRepository = scope.ServiceProvider.GetService<IRepository<Interface_Post_ServiceCode>>();
        //        var _ResultDataConfigInfos = scope.ServiceProvider.GetService<IRepository<Interface_ResultDataConfigInfo>>();
        //        var InterfaceConfig = MESInterfaceConfigService.FindInterfaceConfigDTO(InterfaceType, InterfaceServiceName, default);
        //        var _interface_Request_ConfigRepository = _serviceProvider.GetService<IInterface_Request_ConfigurationRepository>();

        //        DateTime QueryDateEnd = DateTime.Now;
        //        _logger.LogInformation($"正在请求接口[{InterfaceServiceName}]");

        //        var result = await RequestData(MESInterfaceConfigService, InterfaceConfig, RequestURL, QueryDateEnd, RequestDataParameters, EventID);

        //        if (result.code != ExecutionState.SUCCESS)
        //            return result;

        //        var analyticResult = await SavaData(_ResultDataConfigInfos, _interface_Request_ConfigRepository, InterfaceConfig, result.data.ToString(), EventID);
        //        _logger.LogInformation($"接口[{InterfaceServiceName}]请求成功：{analyticResult.message}");
        //        if (string.IsNullOrWhiteSpace(RequestDataParameters) && analyticResult.code == ExecutionState.SaveTooBusinessTableSuccess)
        //        {
        //            if (InterfaceConfig.RequestMethod.ToUpper().Equals("POST"))
        //            {
        //                InterfaceConfig._Post_ServiceCode.LastQuerData = QueryDateEnd;
        //                await __interface_Post_ServicesRepository.UpdateAsync(InterfaceConfig._Post_ServiceCode, true);
        //            }
        //        }

        //        return analyticResult;
        //    }
        //}

        public async Task<ExecutionResult> SingleInterfaceDownload(string InterfaceType, string InterfaceServiceName, string RequestURL = "", string RequestDataParameters = "", string EventID = "")
        {
            var InterfaceConfig = await _interfaceConfigService.FindInterfaceConfigDTO(InterfaceType, InterfaceServiceName, default);

            DateTime QueryDateEnd = DateTime.Now.AddMinutes(-1);

            // var ResultJson = await Download(_interfaceConfigService, InterfaceConfig, QueryDateEnd, RequestDataParameters, EventID);
            var result = await RequestData(_interfaceConfigService, InterfaceConfig, QueryDateEnd,   RequestDataParameters, RequestURL,EventID);

            if (result.code != ExecutionState.SUCCESS)
                return result;

            var analyticResult = await SavaData(_ResultDataConfigInfos, _interface_Request_ConfigRepository, InterfaceConfig, result.data.ToString(), EventID);
            _logger.LogInformation($"EventID:{EventID},接口[{InterfaceServiceName}]请求成功：{analyticResult.message}");
            if (string.IsNullOrWhiteSpace(RequestDataParameters) && analyticResult.code == ExecutionState.SaveTooBusinessTableSuccess)
            {
                if (InterfaceConfig.RequestMethod.ToUpper().Equals("POST"))
                {
                    InterfaceConfig._Post_ServiceCode.LastQuerData = QueryDateEnd;
                    await _interface_Post_ServicesRepository.UpdateAsync(InterfaceConfig._Post_ServiceCode, true);
                }
            }

            return analyticResult;
        }

        ///// <summary>
        ///// 下载数据
        ///// /// </summary>
        ///// <param name="_interfaceConfigService"></param>
        ///// <param name="InterfaceConfig"></param>
        ///// <param name="QueryDateEnd"></param>
        ///// <param name="RequestDataParameters"></param>
        ///// <param name="EventID"></param>
        ///// <returns></returns>
        //private async Task<string> Download(IMESInterfaceConfigService _interfaceConfigService, MESInterfaceConfigDto InterfaceConfig,string RequestURL, DateTime QueryDateEnd, string RequestDataParameters, string EventID = "")
        //{
        //    var requestData = await _interfaceConfigService.BuildRequestDataByInterfaceTypeAsync(InterfaceConfig, RequestURL,QueryDateEnd, RequestDataParameters, default, EventID);

        //    var resultJSON = await Task.Run(() =>
        //    {
        //        _logger.LogInformation($"EventID:{EventID},MES接口类型[{InterfaceConfig.InterfaceType}]正在请求数据。请求数据信息:url:{requestData.Url},请求方法：{InterfaceConfig.RequestMethod},请求参数:{requestData.PostData}");
        //        if (!EventID.IsNullOrEmpty())
        //        {
        //            var _interface_Request_ConfigRepository = _serviceProvider.GetService<IInterface_Request_ConfigurationRepository>();
        //            var sql = @"UPDATE dbo.UploadLog SET jsonData=@json FROM dbo.UploadLog WHERE id=@id";
        //            _interface_Request_ConfigRepository.ExecuteSql(sql, new Microsoft.Data.SqlClient.SqlParameter[] { new Microsoft.Data.SqlClient.SqlParameter("@id", EventID), new Microsoft.Data.SqlClient.SqlParameter("@json", requestData.PostData) });
        //        }
        //        return HttpRequestHelper.AnalysisResponse(HttpRequestHelper.HttpRequest(requestData));
        //    });

        //    return resultJSON;
        //}

        /// <summary>
        /// 下载数据
        /// /// </summary>
        /// <param name="_interfaceConfigService"></param>
        /// <param name="InterfaceConfig"></param>
        /// <param name="QueryDateEnd"></param>
        /// <param name="RequestDataParameters"></param>
        /// <param name="EventID"></param>
        /// <returns></returns>
        private async Task<ExecutionResult> RequestData(IMESInterfaceConfigService _interfaceConfigService, MESInterfaceConfigDto InterfaceConfig, DateTime QueryDateEnd, string RequestDataParameters, string RequestURL = "", string EventID = "")
        {
            var requestData = await _interfaceConfigService.BuildRequestDataByInterfaceTypeAsync(InterfaceConfig, RequestURL,QueryDateEnd, RequestDataParameters, default, EventID);

            _logger.LogInformation($"EventID:{EventID},MES接口类型[{InterfaceConfig.InterfaceType}]正在请求数据。请求数据信息:url:{requestData.Url},请求方法：{InterfaceConfig.RequestMethod},请求参数:{requestData.PostData}");
            if (!EventID.IsNullOrEmpty())
            {
                var _interface_Request_ConfigRepository = _serviceProvider.GetService<IInterface_Request_ConfigurationRepository>();
                var sql = @"UPDATE dbo.UploadLog SET jsonData=@json FROM dbo.UploadLog WHERE id=@id";
                await _interface_Request_ConfigRepository.ExecuteSql(sql, new Microsoft.Data.SqlClient.SqlParameter[] { new Microsoft.Data.SqlClient.SqlParameter("@id", EventID), new Microsoft.Data.SqlClient.SqlParameter("@json", requestData.PostData) });
            }
            return await _httpClient.Request(requestData);
        }

        /// <summary>
        /// 保存数据
        /// </summary>
        /// <param name="_ResultDataConfigInfos"></param>
        /// <param name="_interface_Request_ConfigRepository"></param>
        /// <param name="InterfaceConfig"></param>
        /// <param name="ResultJson"></param>
        /// <param name="EventID"></param>
        /// <returns></returns>
        /// <exception cref="UserFriendlyException"></exception>
        private async Task<ExecutionResult> SavaData(IRepository<Interface_ResultDataConfigInfo> _ResultDataConfigInfos, IInterface_Request_ConfigurationRepository _interface_Request_ConfigRepository,
                                                     MESInterfaceConfigDto InterfaceConfig, string ResultJson, string EventID)
        {
            ExecutionResult analyticResult = new ExecutionResult();

            //拿服务名称
            var InterfaceServiceName = InterfaceConfig.RequestMethod.ToUpper().Equals("POST") ? InterfaceConfig._Post_ServiceCode.InterfaceServiceName : "";

            //拿到接口返回结果配置信息
            var InterfaceRequestDataConfigs = await _ResultDataConfigInfos.GetListAsync(RequestDataConfig =>
                                RequestDataConfig.Interface_ServiceCodeid == InterfaceConfig._Post_ServiceCode.Interface_Post_ServiceCodeID);

            //获取继承的数据
            if (InterfaceRequestDataConfigs.Exists(r => !string.IsNullOrWhiteSpace(r.InheritID)))
            {
                var InheritID = InterfaceRequestDataConfigs.FindLast(r => r.InheritID != "").InheritID;
                InterfaceRequestDataConfigs = await _ResultDataConfigInfos.GetListAsync(RequestDataConfig => RequestDataConfig.Interface_ServiceCodeid == InheritID);
            }

            if (InterfaceRequestDataConfigs == null || InterfaceRequestDataConfigs.Count <= 0)
                throw new UserFriendlyException($"EventID:{EventID},当前接口类型[{InterfaceConfig.InterfaceType}]请求方法是[{InterfaceConfig.RequestMethod}],请求服务[{InterfaceConfig._Post_ServiceCode.InterfaceServiceName}].数据请求成功，但是未配置的返回数据信息", "-1", logLevel: LogLevel.Error);

            var RequestDataConfigs = ObjectMapper.Map<List<Interface_ResultDataConfigInfo>, List<ResultDataConfigInfoDto>>(InterfaceRequestDataConfigs);

            JToken Roottoken = null;
            try
            {
                Roottoken = JsonHelper.ReadJSON(ResultJson);
            }
            catch (Exception ex)
            {
                throw new UserFriendlyException($"EventID:{EventID},接口返回信息解析出错。错误原因:{ex.Message},数据:{ResultJson}");
            }

            //拿到返回的数据并放入对象
            RequestDataConfigs.ForEach(r =>
            {
                if (ResultJson.Contains(r.ResultKeyName))
                {
                    if (!string.IsNullOrEmpty(InterfaceConfig._Post_ServiceCode.ResultObjectName))
                    {
                        try
                        {
                            r.Value = Roottoken[InterfaceConfig._Post_ServiceCode.ResultObjectName].SelectToken(r.ResultKeyName)?.ToString();
                        }
                        catch (Exception ex)
                        {
                            throw new UserFriendlyException($"EventID:{EventID},接口返回信息json节点[{InterfaceConfig._Post_ServiceCode.ResultObjectName}]下没有字段名称:[{r.ResultKeyName}],对方接口返回信息：{ResultJson}");
                        }
                    }
                    else
                        r.Value = Roottoken.SelectToken(r.ResultKeyName).ToString();
                }
                else
                    throw new UserFriendlyException($"EventID:{EventID},当前接口类型[{InterfaceConfig.InterfaceType}]请求方法是[{InterfaceConfig.RequestMethod}],请求服务[{InterfaceConfig._Post_ServiceCode.InterfaceServiceName}].数据请求成功，但是配置的返回类型[{r.ResultKeyName}]在返回数据中不存在{ResultJson}", "-1", logLevel: LogLevel.Error);
            });

            //拿状态对象
            var statusRDCDTO = RequestDataConfigs.Find(rdc => rdc.KeyType == KeyTypeInfo.Status.ToString());

            //拿返回的消息
            var message = RequestDataConfigs.Find(rdc => rdc.KeyType == KeyTypeInfo.message.ToString()).Value;

            var LINES = "0";
            //拿统计信息
            var StatisticDto = RequestDataConfigs.Find(rdc => rdc.KeyType == KeyTypeInfo.Statistic.ToString());

            if (StatisticDto != null)
                LINES = StatisticDto?.Value;

            // analyticResult.details = ResultJson;
            analyticResult.message = message;
            analyticResult.data = JsonHelper.DeserializeStringToObject(ResultJson);

            //判断状态
            if (statusRDCDTO.Value.ToUpper() != statusRDCDTO.KeyValue.ToUpper())
            {
                _logger.LogInformation($"EventID:{EventID},接口{InterfaceServiceName}请求失败:{message}");
                analyticResult.code = ExecutionState.FAIL;

                return analyticResult;
            }
            else
            {
                analyticResult.code = ExecutionState.SUCCESS;
            }
            //如果统计对象不会空就判断是否拿到正确的数据
            if (StatisticDto != null)
            {
                if (statusRDCDTO.Value.ToUpper() == statusRDCDTO.KeyValue.ToUpper() && int.Parse(LINES) == 0)
                {
                    _logger.LogInformation($"EventID:{EventID},接口{InterfaceServiceName}请求成功但是没有获取到数据:{message}");

                    analyticResult.code = ExecutionState.SaveTooBusinessTableSuccess;
                    analyticResult.message = message;
                    return analyticResult;
                }
                _logger.LogInformation($"EventID:{EventID},接口{InterfaceServiceName}请求成功获取到{LINES}条数据");
            }

            var id = Guid.NewGuid().ToString();
            _logger.LogInformation($"EventID:{EventID},接口{InterfaceServiceName}中间表表组ID:{id}创建成功");

            //创建表
            RequestDataConfigs.FindAll(rdc => rdc.KeyType == KeyTypeInfo.data.ToString() && rdc.IsCreateTable).ForEach(rdc =>
            {
                JArray etDataList = null;
                if (!string.IsNullOrEmpty(rdc.Value))
                {
                    if (!string.IsNullOrEmpty(rdc.DataObjectName))
                    {
                        var JSONObj = JsonHelper.ReadJSON(rdc.Value);
                        etDataList = JsonHelper.GetToJsonList(JSONObj.SelectToken(rdc.DataObjectName).ToString());
                    }
                    else
                        etDataList = JsonHelper.GetToJsonList(rdc.Value);
                }
                else
                {
                    etDataList = JsonHelper.GetToJsonList("[]");
                }
                rdc.tableModel = CreateDataTable(_interface_Request_ConfigRepository, InterfaceServiceName + "_" + rdc.ResultKeyName, etDataList, id).Result;
                if (rdc.tableModel == null)
                {
                    _logger.LogInformation($"EventID:{EventID},未获取到[{rdc.ResultKeyName}]的数据,停止创建表");
                }
                else
                {
                    _logger.LogInformation($"EventID:{EventID},表[{rdc.tableModel.TableName}]状态:[{rdc.tableModel.CreateTableStatus.ToString()}],准备写入数据");

                    if (rdc.tableModel.CreateTableStatus != CreateTableStatus.Fail)
                    {
                        _interface_Request_ConfigRepository.BulkCopy(rdc.tableModel.Data).Wait();
                        _logger.LogInformation($"EventID:{EventID},表[{rdc.tableModel.TableName}]数据写入成功，共计[{rdc.tableModel.Data.Rows.Count}]条记录");
                    }
                }
            });

            var CreateTableFailList = RequestDataConfigs.FindAll(rdc => rdc.IsCreateTable && rdc.tableModel != null && rdc.tableModel.CreateTableStatus == CreateTableStatus.Fail);
            if (CreateTableFailList.Count > 0)
            {
                RequestDataConfigs.FindAll(rdc => rdc.KeyType == KeyTypeInfo.data.ToString() && rdc.IsCreateTable).ForEach(rdc =>
                {
                    analyticResult.message += $"EventID:{EventID},接口请求成功但是表:{rdc.tableModel.TableName}创建失败:{rdc.tableModel.CreateTableFailMSG}";
                    _interface_Request_ConfigRepository.ExecuteSql($"delete [{rdc.tableModel.TableName}] where GroupId='{id}' ").Wait();
                    _logger.LogInformation($"EventID:{EventID},接口请求成功但是表:{rdc.tableModel.TableName}创建失败:{rdc.tableModel.CreateTableFailMSG}");
                });

                analyticResult.code = ExecutionState.FAIL;
                return analyticResult;
            }

            var MasterData = RequestDataConfigs.Find(rdc => rdc.IsMaster);

            //执行存储过程把数据保存到业务表
            if (MasterData != null && MasterData.tableModel != null && MasterData.tableModel.CreateTableStatus != CreateTableStatus.Fail)
            {
                var sql = $@"if exists(select * from dbo.sysobjects where id = object_id(N'[dbo].[{InterfaceConfig._Post_ServiceCode.StoredProcedureToSaveDataToBusinessTable}]') and OBJECTPROPERTY(id, N'IsProcedure') = 1) begin
	                       select 'exists' as ResultMSG
                            end
                            else
                             select 'no exists' as ResultMSG";

                var ds = await _interface_Request_ConfigRepository.ExecuteSql(sql);
                if (ds != null && ds.Tables.Count > 0 && ds.Tables[ds.Tables.Count - 1].Rows.Count > 0 && ds.Tables[ds.Tables.Count - 1].Rows[0][0].ToString().ToUpper().Equals("NO EXISTS"))
                {
                    throw new UserFriendlyException($"EventID:{EventID},当前接口服务类型:{InterfaceConfig._Post_ServiceCode.InterfaceServiceName}请求方法:{InterfaceConfig.RequestMethod}，数据请求成功并保存到中间表但是保存数据到业务表的存储过程[{InterfaceConfig._Post_ServiceCode.StoredProcedureToSaveDataToBusinessTable}]不存在！", "-1", logLevel: LogLevel.Error);
                    //   _logger.LogInformation($"当前接口服务类型:{InterfaceConfig._Post_ServiceCode.InterfaceServiceName}请求方法:{InterfaceConfig.RequestMethod}，数据请求成功并保存到中间表但是保存数据到业务表的存储过程[{InterfaceConfig._Post_ServiceCode.StoredProcedureToSaveDataToBusinessTable}]不存在！");
                }

                _logger.LogInformation($"EventID:{EventID},{InterfaceServiceName}数据准备写入到业务表。保存数据到业务表的存储过程:{InterfaceConfig._Post_ServiceCode.StoredProcedureToSaveDataToBusinessTable}");
                analyticResult.code = ExecutionState.SaveDataSuccess;
                var resultds = await _interface_Request_ConfigRepository.ExecuteStoredProcedureResultDataSet(InterfaceConfig._Post_ServiceCode.StoredProcedureToSaveDataToBusinessTable,
                                                                                  new Microsoft.Data.SqlClient.SqlParameter[] {
                                                                                        new Microsoft.Data.SqlClient.SqlParameter("@FactoryId" , InterfaceConfig.FactoryId),
                                                                                        new Microsoft.Data.SqlClient.SqlParameter("@Groupid" , MasterData.tableModel.GroupIDValue) });

                bool SavaState = true;
                if (resultds != null && resultds.Tables.Count > 0 && resultds.Tables[resultds.Tables.Count - 1].Rows.Count > 0)
                {
                    analyticResult.message = $"EventID:{EventID},接口{InterfaceServiceName}数据写入业务表:{resultds.Tables[resultds.Tables.Count - 1].Rows[0]["ReturnMSG"].ToString()}";
                    if (resultds.Tables[resultds.Tables.Count - 1].Rows[0]["ReturnValue"].ToString() == "0")
                    {
                        analyticResult.code = ExecutionState.SaveTooBusinessTableSuccess;
                        analyticResult.details = "";
                    }
                    if (SavaState && resultds.Tables[resultds.Tables.Count - 1].Rows[0]["ReturnValue"].ToString() == "-1")
                    {
                        analyticResult.message = resultds.Tables[resultds.Tables.Count - 1].Rows[0]["ReturnMSG"].ToString();
                        _logger.LogInformation($"EventID:{EventID},{InterfaceServiceName}数据写入业务表失败。存储过程:{InterfaceConfig._Post_ServiceCode.StoredProcedureToSaveDataToBusinessTable}，原因:{resultds.Tables[resultds.Tables.Count - 1].Rows[0]["ReturnMSG"].ToString()}");
                    }
                }
                else
                {
                    analyticResult.message = $"EventID:{EventID},{InterfaceServiceName}数据写入业务表失败。存储过程:{InterfaceConfig._Post_ServiceCode.StoredProcedureToSaveDataToBusinessTable}未返回数据";
                    //    _logger.LogInformation($"{InterfaceServiceName}数据写入业务表失败。存储过程:{InterfaceConfig._Post_ServiceCode.StoredProcedureToSaveDataToBusinessTable}未返回数据");
                }
            }

            return analyticResult;
        }

        private async Task<TableModel> CreateDataTable(IInterface_Request_ConfigurationRepository _interface_Request_ConfigRepository, string TableName, JArray jlist, string id = "")
        {
            if (jlist.Count <= 0)
                return null;
            var model = CreateTableHelper.BuildTableModel(TableName, jlist, id);

            var ds = await _interface_Request_ConfigRepository.ExecuteSql(CreateTableHelper.CheckTableExists);
            if (ds != null && ds.Tables.Count > 0 && ds.Tables[ds.Tables.Count - 1].Rows.Count > 0 && ds.Tables[ds.Tables.Count - 1].Rows[0][0].ToString().ToUpper().Equals("N"))
            {
                var sql = CreateTableHelper.BulidCreateTableSQL(model);
                _logger.LogInformation($"正在创建表[{TableName}],sql;{sql}");
                await _interface_Request_ConfigRepository.ExecuteSql(sql);

                model.CreateTableStatus = CreateTableStatus.Success;
            }
            else if (ds != null && ds.Tables.Count > 0 && ds.Tables[ds.Tables.Count - 1].Rows.Count > 0 &&
                     ds.Tables[ds.Tables.Count - 1].Rows[0][0].ToString().ToUpper().Equals("Y"))
            {
                model.CreateTableStatus = CreateTableStatus.Exist;
                var dbinfo = await _interface_Request_ConfigRepository.QueryDBTableInfo(CreateTableHelper.QueryDBTableInfo);
                CreateTableHelper.TableDiff(model, dbinfo.ToList());
                if (model.AlterColumnDescription.Count > 0)
                {
                    var sql = CreateTableHelper.BulidNewFieldSQL(model);
                    await _interface_Request_ConfigRepository.ExecuteSql(sql);
                }
            }

            return model;
        }
    }
}