﻿using Microsoft.Data.SqlClient;
using Microsoft.Extensions.Logging;
using Microsoft.Identity.Client;
using Newtonsoft.Json;
using OrBit.MESInterface.Application.Contracts.Dtos;
using OrBit.MESInterface.Application.Contracts.Interfaces;
using OrBit.MESInterface.Application.Contracts.Interfaces.Dtos;
using OrBit.MESInterface.Domain.InterfaceConfiguration.Entities;
using OrBit.MESInterface.Domain.InterfaceConfiguration.Repository;
using OrBit.MESInterface.Domain.SysPlugIn.Repository;
using OrBit.MESInterface.Infrastructure.Common;
using OrBit.MESInterface.Infrastructure.CreateTable;
using OrBit.MESInterface.Infrastructure.DataSetToJson;
using OrBit.MESInterface.Infrastructure.Extensions;
using OrBit.MESInterface.Infrastructure.Jwt;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Data;
using System.Data.Common;
using System.Linq;
using System.Linq.Dynamic.Core;
using System.Threading;
using System.Threading.Tasks;
using Volo.Abp;
using Volo.Abp.Application.Services;
using LogLevel = Microsoft.Extensions.Logging.LogLevel;

namespace OrBit.MESInterface.Application.Service
{
    public class MESInterfaceConfigService : ApplicationService, IMESInterfaceConfigService
    {
        private readonly IInterface_Request_ConfigurationRepository _interface_Request_ConfigRepository;
        private readonly ILogger<MESInterfaceConfigService> _logger;
        private readonly ISysParameterRepository sysParameter;

        public MESInterfaceConfigService(IInterface_Request_ConfigurationRepository interface_Request_ConfigurationRepository, ILogger<MESInterfaceConfigService> logger, ISysParameterRepository sysParameter)
        {
            _interface_Request_ConfigRepository = interface_Request_ConfigurationRepository;
            _logger = logger;
            this.sysParameter = sysParameter;
        }

        public async Task<Interface_Request_Configuration> FindInterfaceConfig(string InterfaceType, CancellationToken cancellationToken)
        {
            var Interface_Request_Config = await _interface_Request_ConfigRepository.GetAsync(Interface_Request_Config => Interface_Request_Config.InterfaceType.Equals(InterfaceType)
                                                        , true, cancellationToken);

            return Interface_Request_Config;
        }

        public async Task<MESInterfaceConfigDto> FindInterfaceConfigDTO(string InterfaceType, string InterfaceServiceName, CancellationToken cancellationToken = default)
        {
            var Interface_Request_Config = await _interface_Request_ConfigRepository.GetAsync(Interface_Request_Config => Interface_Request_Config.InterfaceType.Equals(InterfaceType) && Interface_Request_Config.IsDel.Equals(false)
                                                         , true, cancellationToken);

            if (Interface_Request_Config == null)
                throw new UserFriendlyException($"没有获取到接口类型[{InterfaceType}]对应的配置数据", "-1", logLevel: LogLevel.Error);

            if (Interface_Request_Config.RequestMethod.ToLower() == "post" && (Interface_Request_Config._Post_ServiceCodes == null || Interface_Request_Config._Post_ServiceCodes.ToList().Count <= 0))
                throw new UserFriendlyException($"当前接口[{InterfaceType}]类型{Interface_Request_Config.InterfaceType_Describe}请求方法是{Interface_Request_Config.RequestMethod}但是没有获取到对应的接口服务配置数据", "-1", logLevel: LogLevel.Error);

            //if (Interface_Request_Config.RequestMethod.ToLower() == "get" &&  (Interface_Request_Config._g == null || Interface_Request_Config._Post_ServiceCodes.ToList().Count <= 0))
            //    throw new ArgumentNullException("InterfaceType", $"没有获取到接口类型{InterfaceType}对应的接口服务配置数据");

            var InterfaceConfig = ObjectMapper.Map<Interface_Request_Configuration, MESInterfaceConfigDto>(Interface_Request_Config);

            if (InterfaceConfig.RequestMethod.ToLower() == "post")
            {
                var _Post_ServiceCode = Interface_Request_Config._Post_ServiceCodes.ToList().Find(psc => psc.InterfaceServiceName.Equals(InterfaceServiceName));

                if (_Post_ServiceCode == null)
                    throw new UserFriendlyException($"当前接口[{InterfaceType}]类型{Interface_Request_Config.InterfaceType_Describe}请求方法是{Interface_Request_Config.RequestMethod}但是没有获取到对应的接口服务[{InterfaceServiceName}]的配置数据",
                                                   "-1", logLevel: LogLevel.Error);

                if (_Post_ServiceCode.IsReplaceInInterfaceServiceField)
                {
                    InterfaceConfig.RequestURL = InterfaceConfig.RequestURL.Replace($"{"{"}{_Post_ServiceCode.InterfaceServiceReplacementField}{"}"}", _Post_ServiceCode.InterfaceServiceName);
                }

                InterfaceConfig._Post_ServiceCode = _Post_ServiceCode;
            }
            return InterfaceConfig;
        }

        public MESInterfaceConfigDto FindInterfaceConfigDTO(Interface_Request_Configuration Interface_Request_Config, string InterfaceServiceName, CancellationToken cancellationToken = default)
        {
            if (Interface_Request_Config == null)
                throw new UserFriendlyException($"没有获取到接口配置数据", "-1", logLevel: LogLevel.Error);

            if (Interface_Request_Config.RequestMethod.ToLower() == "post" && (Interface_Request_Config._Post_ServiceCodes == null || Interface_Request_Config._Post_ServiceCodes.ToList().Count <= 0))
                throw new UserFriendlyException($"当前接口[{Interface_Request_Config.InterfaceType}]类型{Interface_Request_Config.InterfaceType_Describe}请求方法是{Interface_Request_Config.RequestMethod}但是没有获取到对应的接口服务配置数据", "-1", logLevel: LogLevel.Error);

            //if (Interface_Request_Config.RequestMethod.ToLower() == "get" &&  (Interface_Request_Config._g == null || Interface_Request_Config._Post_ServiceCodes.ToList().Count <= 0))
            //    throw new ArgumentNullException("InterfaceType", $"没有获取到接口类型{InterfaceType}对应的接口服务配置数据");

            var InterfaceConfig = ObjectMapper.Map<Interface_Request_Configuration, MESInterfaceConfigDto>(Interface_Request_Config);

            if (InterfaceConfig.RequestMethod.ToLower() == "post")
            {
                var _Post_ServiceCode = Interface_Request_Config._Post_ServiceCodes.ToList().Find(psc => psc.InterfaceServiceName.Equals(InterfaceServiceName));

                if (_Post_ServiceCode == null)
                    throw new UserFriendlyException($"当前接口[{Interface_Request_Config.InterfaceType}]类型{Interface_Request_Config.InterfaceType_Describe}请求方法是{Interface_Request_Config.RequestMethod}但是没有获取到对应的接口服务[{InterfaceServiceName}]的配置数据",
                                                   "-1", logLevel: LogLevel.Error);

                if (_Post_ServiceCode.IsReplaceInInterfaceServiceField)
                {
                    InterfaceConfig.RequestURL = InterfaceConfig.RequestURL.Replace($"{"{"}{_Post_ServiceCode.InterfaceServiceReplacementField}{"}"}", _Post_ServiceCode.InterfaceServiceName);
                }

                InterfaceConfig._Post_ServiceCode = _Post_ServiceCode;
            }
            return InterfaceConfig;
        }

        public async Task<RequestData> BuildRequestDataByInterfaceTypeAsync(MESInterfaceConfigDto InterfaceConfig, string RequestURL, DateTime QueryDateEnd, string RequestDataParameters, CancellationToken cancellationToken, string EventID)
        {
            var PostData = string.Empty;
            if (InterfaceConfig.RequestMethod.ToLower() == "post")
            {
                if (InterfaceConfig._Post_ServiceCode == null)
                    throw new UserFriendlyException($"EventID:{EventID},当前接口服务类型{InterfaceConfig._Post_ServiceCode.InterfaceServiceName}请求方法是{InterfaceConfig.RequestMethod}但是没有获取到对应的Post请求配置信息", "-1", logLevel: LogLevel.Error);

                if (string.IsNullOrEmpty(InterfaceConfig._Post_ServiceCode.FillParameterTemplateByStoredProcedure))
                    throw new UserFriendlyException($"EventID:{EventID},当前接口服务类型{InterfaceConfig._Post_ServiceCode.InterfaceServiceName}请求方法是{InterfaceConfig.RequestMethod}但是没有获取到对应的Post请求参数模板配置信息", "-1", logLevel: LogLevel.Error);

                if (string.IsNullOrEmpty(InterfaceConfig._Post_ServiceCode.ParameterTemplate))
                    throw new UserFriendlyException($"EventID:{EventID},当前接口服务类型{InterfaceConfig._Post_ServiceCode.InterfaceServiceName}请求方法是{InterfaceConfig.RequestMethod}但是没有获取到对应的Post请求参数模板", "-1", logLevel: LogLevel.Error);

                if (string.IsNullOrEmpty(InterfaceConfig._Post_ServiceCode.FillParameterTemplateByStoredProcedure))
                    throw new UserFriendlyException($"EventID:{EventID},当前接口服务类型{InterfaceConfig._Post_ServiceCode.InterfaceServiceName}请求方法是{InterfaceConfig.RequestMethod}但是没有获取到对应的Post请求参数填充模板存储过程", "-1", logLevel: LogLevel.Error);

                if (string.IsNullOrEmpty(InterfaceConfig._Post_ServiceCode.StoredProcedureToSaveDataToBusinessTable))
                    throw new UserFriendlyException($"EventID:{EventID},当前接口服务类型{InterfaceConfig._Post_ServiceCode.InterfaceServiceName}请求方法是{InterfaceConfig.RequestMethod}但是没有获取到对应的Post请求保存数据到业务表的存储过程", "-1", logLevel: LogLevel.Error);

                PostData = await GetDownloadlDataForFillParameterTemplate(InterfaceConfig._Post_ServiceCode.ParameterTemplate, InterfaceConfig._Post_ServiceCode.FillParameterTemplateByStoredProcedure, InterfaceConfig._Post_ServiceCode.InterfaceServiceName, InterfaceConfig.Interface_Request_ConfigurationID, QueryDateEnd, RequestDataParameters, EventID);
            }

            var RequestHeaders = new Dictionary<string, string>();
            var headers = InterfaceConfig._RequestHeaders.Where(h => !h.IsDel)?.ToList();
            if (headers.Count > 0)
            {
                for (var i = 0; i < headers.Count; i++)
                {
                    await BuildHeaders(headers[i], EventID);
                }
                RequestHeaders = headers.Select(p => new { p.RequestHeaderName, p.RequestHeaderValue })
                    .ToDictionary(RequestHeader => RequestHeader.RequestHeaderName, RequestHeader => RequestHeader.RequestHeaderValue);
            }

            //先取接口配置请求地址
            var _requestURL = InterfaceConfig.RequestURL;

            //如果存在自定义请求地址优先使用自定义地址
            if(!string.IsNullOrWhiteSpace(RequestURL))
            {
                _requestURL = RequestURL;
            }

            var _RequestData = new RequestData(_requestURL, InterfaceConfig.RequestMethod, InterfaceConfig.IsBasicAuth, InterfaceConfig.AuthUserName,
                                                InterfaceConfig.AuthPassword, null, PostData, RequestHeaders
                                               );

            return _RequestData;
        }

        private async Task BuildHeaders(Interface_RequestHeader header, string eventID)
        { 
            if (!string.IsNullOrWhiteSpace(header.HeaderValueType)&& header.HeaderValueType.Equals("BearerToken"))
            {
                var parameterList = await sysParameter.GetListAsync(p => p.ParameterName.Equals("[MESInterface]"));
                var issuer = parameterList.Where(p => p.ParameterValue.Equals("issuer")).FirstOrDefault()?.ParameterValueEx;
                var jwtSecret = parameterList.Where(p => p.ParameterValue.Equals("JwtSecret")).FirstOrDefault()?.ParameterValueEx;
                var userName = parameterList.Where(p => p.ParameterValue.Equals("UserName")).FirstOrDefault()?.ParameterValueEx;
                if (string.IsNullOrWhiteSpace(issuer))
                    throw new UserFriendlyException($"EventID:{eventID},当前接口配置了请求头为Bearer Token但是没有获取到对应的issuer配置信息", "-1", logLevel: LogLevel.Error);
                if (string.IsNullOrWhiteSpace(jwtSecret))
                    throw new UserFriendlyException($"EventID:{eventID},当前接口配置了请求头为Bearer Token但是没有获取到对应的jwtSecret配置信息", "-1", logLevel: LogLevel.Error);
                if (string.IsNullOrWhiteSpace(userName))
                    throw new UserFriendlyException($"EventID:{eventID},当前接口配置了请求头为Bearer Token但是没有获取到对应的userName配置信息", "-1", logLevel: LogLevel.Error);

                var jwtTool = new JwtTool(new JwtBearerSetting(issuer, issuer, jwtSecret));
                var token = jwtTool.CustomizationGenerateAccessToken(userName);

                header.RequestHeaderValue = token;
            }
        }

        /// <summary>
        /// 获取填充参数模板的数据
        /// </summary>
        /// <param name="JsonTemplate">请求参数模板</param>
        /// <param name="FillParameterTemplateByStoredProcedure">获取填充参数模板数据的存储过程</param>
        /// <param name="InterfaceServiceName">接口服务</param>
        /// <param name="Interface_Request_ConfigurationID">接口类型ID</param>
        /// <param name="QueryParameters"></param>
        /// <returns></returns>
        private async Task<string> GetDownloadlDataForFillParameterTemplate(string JsonTemplate, string FillParameterTemplateByStoredProcedure, string InterfaceServiceName, string Interface_Request_ConfigurationID, DateTime QueryDateEnd, string QueryParameters, string EventID)
        {
            //SqlParameter[] paras = { new SqlParameter("@serviceCode", serviceCode), new SqlParameter("@DateEnd", QueryTime), new SqlParameter("@QueryParameters", QueryParameters) };

            var sql = $@"if exists(select * from dbo.sysobjects where id = object_id(N'[dbo].[{FillParameterTemplateByStoredProcedure}]') and OBJECTPROPERTY(id, N'IsProcedure') = 1) begin
	                       select 'exists' as ResultMSG
                            end
                            else
                             select 'no exists' as ResultMSG";

            var ds = await _interface_Request_ConfigRepository.ExecuteSql(sql);
            if (ds != null && ds.Tables.Count > 0 && ds.Tables[ds.Tables.Count - 1].Rows.Count > 0 && ds.Tables[ds.Tables.Count - 1].Rows[0][0].ToString().ToUpper().Equals("no exists"))
            {
                throw new UserFriendlyException($"EventID:{EventID},当前接口服务类型:{InterfaceServiceName}，获取请求参数的存储过程[{FillParameterTemplateByStoredProcedure}]不存在！", "-1", logLevel: LogLevel.Error);
            }

          //  _logger.LogInformation($"接口{InterfaceServiceName},QueryDateEnd:{QueryDateEnd}");
            var result = await _interface_Request_ConfigRepository.ExecuteStoredProcedureResultDataSet(FillParameterTemplateByStoredProcedure,
                                               new SqlParameter[] { new SqlParameter ("@Interface_Request_ConfigurationID", Interface_Request_ConfigurationID ),
                                                             new SqlParameter("@InterfaceServiceName", InterfaceServiceName ),
                                                             new SqlParameter("@QueryDateEnd",QueryDateEnd.ToString("yyyy-MM-dd HH:mm:ss")),
                                                             new SqlParameter ("@QueryParameters", QueryParameters) });

            if (result == null || result.Tables.Count <= 0 || result.Tables[0].Rows.Count <= 0)
                throw new UserFriendlyException($"EventID:{EventID},当前接口服务类型:{InterfaceServiceName}，没有获取到请求参数", "-1", logLevel: LogLevel.Error);

            IDataSet2JsonService dataSet2Json = new DefaultDataSet2JsonService();
            var jsonType = JsonHelper.isJArray(JsonTemplate) ? "[]" : "{}";
            return dataSet2Json.BuildJSON(result, jsonType);
        }
    }
}