﻿using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using OrBit.MESInterface.Application.Contracts.MESInterface;
using OrBit.MESInterface.Domain.InterfaceConfiguration.Repository;
using OrBit.MESInterface.Domain.MesInterfaceConfiguration.Repository;
using OrBit.MESInterface.Infrastructure;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using Volo.Abp.Application.Services;
using Volo.Abp.Domain.Repositories;
using static System.Net.Mime.MediaTypeNames;

namespace OrBit.MESInterface.Application.Service.MESInterface
{
    public class OrBitAuthorizationService : ApplicationService, IOrBitAuthorizationService
    {
        private readonly ILogger<OrBitAuthorizationService> _logger;

        private readonly IMESInterfaceConifgRepository db;

        public OrBitAuthorizationService(ILogger<OrBitAuthorizationService> logger, IMESInterfaceConifgRepository _db)
        {
            _logger = logger;
            db = _db;
        }

        public async Task<ExecutionResult> Authorization(string access_token, string Timestamp, string RequestBody, string ConsumerId, string Authorization)
        {
            var AnalyticResult = new ExecutionResult(ExecutionState.FAIL, "认证失败");
            var mc = await db.SingleOrDefaultAsync(m => m.ConsumerId.Equals(ConsumerId));
            if (mc == null)
                return ExecutionResult.Failed("缺少参数:ConsumerId");

            if (mc.PermissionAuthenticationMethod.Contains("PrivacyToken"))
            {
                var ConsumerSecret = mc.ConsumerSecret;
                // var str = ConsumerSecret + Timestamp;

                var str = ConsumerSecret + Convert.ToBase64String(Encoding.Default.GetBytes(RequestBody)) + Timestamp;
                var md5Token = GetMD5Hash(str);

                if (md5Token.Equals(access_token))
                {
                    AnalyticResult.code = ExecutionState.SUCCESS;
                    AnalyticResult.message = "";
                }
            }
            if (mc.PermissionAuthenticationMethod.Contains("BasicAuth"))
            {
                // 判断是否加上了不需要拦截
                string authHeader = Authorization;

                if (authHeader != null && authHeader.StartsWith("Basic"))
                {
                    //Extract credentials
                    string encodedUsernamePassword = authHeader.Substring("Basic ".Length).Trim();
                    Encoding encoding = Encoding.GetEncoding("iso-8859-1");
                    string usernamePassword = encoding.GetString(Convert.FromBase64String(encodedUsernamePassword));
                    int seperatorIndex = usernamePassword.IndexOf(':');
                    var username = usernamePassword.Substring(0, seperatorIndex);
                    var password = usernamePassword.Substring(seperatorIndex + 1);

                    //if (IsAuthorized(username, password))
                    if (mc.AuthUserName.Equals(username) && mc.AuthPassword.Equals(password))
                    {
                        AnalyticResult.code = ExecutionState.SUCCESS;
                        AnalyticResult.message = "";
                    }
                }
            }

            return AnalyticResult;
        }

        public string GetMD5Hash(string str)
        {//32位大写
            using (var md5 = MD5.Create())
            {
                var result = md5.ComputeHash(Encoding.UTF8.GetBytes(str));
                var strResult = BitConverter.ToString(result);
                return strResult.Replace("-", "");
            }
        }
    }
}