﻿using Microsoft.AspNetCore.Http;
using Microsoft.Data.SqlClient;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using OrBit.MESInterface.Application.Contracts.MESInterface;
using OrBit.MESInterface.Application.Contracts.MESInterface.Dto;
using OrBit.MESInterface.Domain.MesInterfaceConfiguration.Entities;
using OrBit.MESInterface.Domain.MesInterfaceConfiguration.Repository;
using OrBit.MESInterface.Infrastructure;
using OrBit.MESInterface.Infrastructure.Common;
using OrBit.MESInterface.Infrastructure.CreateTable;
using OrBit.MESInterface.Infrastructure.DataSetToJson;
using OrBit.MESInterface.Infrastructure.Extensions;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Threading.Tasks;
using OrBit.MESInterface.Application.Contracts.ApiFile;
using OrBit.MESInterface.Application.Contracts.ApiFile.Dtos;
using Volo.Abp.Application.Services;
using Volo.Abp.Domain.Repositories;
using static System.Runtime.InteropServices.JavaScript.JSType;

namespace OrBit.MESInterface.Application.Service.MESInterface
{
    public class PutDataAppService : ApplicationService, IPutDataAppService
    {
        private readonly ILogger<PutDataAppService> _logger;
        private IMESInterfaceConifgRepository _mewConfigService;
        private IServiceProvider serviceProvider;

        private readonly IAPIFileAppService apiFileAppService;

        public PutDataAppService(IServiceProvider serviceProvider, ILogger<PutDataAppService> logger, IMESInterfaceConifgRepository mewConfigService, IAPIFileAppService apiFileAppService)
        {
            _logger = logger;
            _mewConfigService = mewConfigService;
            this.serviceProvider = serviceProvider;
            this.apiFileAppService = apiFileAppService;
        }

        public async Task<ExecutionResult> PutFileDataSaveAsync(HttpContext httpContext, string ConsumerId, string id = "")
        {
            if (!httpContext.Request.Form.ContainsKey("InterfaceServiceName") && !httpContext.Request.Form.ContainsKey("INTERFACESERVICENAME"))
            {
                return ExecutionResult.Failed("当前请求没有给定服务名称！");
            }
            var InterfaceServiceName = httpContext.Request.Form["InterfaceServiceName"];
            var app = "MESInterface";
            //验证服务
            var result = await VerifyService(ConsumerId, InterfaceServiceName);

            if (result.code != ExecutionState.SUCCESS)
            {
                Logger.LogInformation($"请求事件id:{id},服务名称:{InterfaceServiceName}验证失败，失败原因:{result.message}");
                return result;
            }
            
            var InterfaceConfig = result.details as MESInterfaceConfigInfo;
            var ServiceCofig = result.data as MESInterfaceServiceConfigInfo;

            if (!InterfaceConfig.ActivationUpload)
                return ExecutionResult.Failed("当前消费方没有上传文件权限");

            //验证参数
            var VerifyParameterList = await VerifyParameters(ServiceCofig, param =>
            {
                if (httpContext.Request.Form.ContainsKey(param))
                    return ExecutionResult.Success("", httpContext.Request.Form[param]);
                return ExecutionResult.Failed("没有值");
            });

            if (VerifyParameterList.code != ExecutionState.SUCCESS)
            {
                Logger.LogInformation($"请求事件id:{id},服务名称:{InterfaceServiceName},参数验证失败，失败原因:{VerifyParameterList.message}");

                return ExecutionResult.Failed(VerifyParameterList.message, "E");
            }

            List<MESInterfaceParametersConfigDto> parameters = VerifyParameterList.data == null ? new List<MESInterfaceParametersConfigDto>() : VerifyParameterList.data as List<MESInterfaceParametersConfigDto>;
            var logInfo = "";
            parameters.ForEach(p =>
            {
                logInfo += $"{p.ParameterName}:{p.ParameterValue};";
            });
            //判断执行脚本在不在
            var VerificationScriptresult = await VerificationScript(ServiceCofig, id);
            if (VerificationScriptresult.code == ExecutionState.FAIL)
            {
                VerificationScriptresult.data = "E";
                return VerificationScriptresult;
            }

            //校验执行
            if (ServiceCofig.InterfaceType.Equals("推送数据与文件"))
            {
                var log_FileInfo = "";
                httpContext.Request.Form.Files.ToList().ForEach(f =>
                {
                    log_FileInfo += $"文件名：{f.FileName},大小：{f.Length / 1024.0 / 1024.0}.mb";
                });

                _logger.LogInformation($"请求事件id:{id},接口名称：{InterfaceServiceName}参数:{logInfo},文件信息：{log_FileInfo}");


                //上传文件
                var uploadFileResult = await apiFileAppService.UploadAndSaveFileInfo(new UplaodFileInfoDto
                {
                    app = app,
                    files = httpContext.Request.Form.Files,
                    UploadFolder = InterfaceConfig.UploadFolder,
                    UserName = InterfaceConfig.ConsumerId
                });

                if (uploadFileResult.code == ExecutionState.FAIL)
                {
                    _logger.LogInformation($"请求事件id:{id},文件保存失败,失败原因：{uploadFileResult.message}");
                    return uploadFileResult;
                }
                    
                _logger.LogInformation($"请求事件id:{id},文件保存完成");
               var newFiles = JsonConvert.DeserializeObject<List<FileInfoDto>>(uploadFileResult.DataToJson());// uploadFileResult.data.ConvertToDesiredType<FileInfoDto>();
                var fileids = "";
                newFiles.ForEach(f => fileids += f.FileID+";");
                //执行脚本
                var ExecResult = await ExecScrip(ServiceCofig, parameters, fileids);

               var analyticResult= await AnalyticalResults(ServiceCofig, ExecResult);
                Logger.LogInformation($"当前请求事件[{id}],服务名称:{InterfaceServiceName},执行结果:{analyticResult.code},返回信息：{analyticResult.message}");
                //解析验证脚本结果
                return analyticResult;
            }

            return ExecutionResult.Failed("不推送文件时请修改Content-Typet值为application/json");
        }
        public async Task<ExecutionResult> PutJsonDataSaveAsync(string InterfaceServiceName, string data, string ConsumerId, HttpContext httpContext, string id = "")
        {
            ExecutionResult analyticResult = new ExecutionResult();

            //验证服务
            var result = await VerifyService(ConsumerId, InterfaceServiceName);
            if (result.code != ExecutionState.SUCCESS)
            {
                Logger.LogInformation($"请求事件id:{id},服务名称:{InterfaceServiceName}验证失败，失败原因:{result.message}");
                return result;
            }

            var ServiceCofig = result.data as MESInterfaceServiceConfigInfo;

            //验证参数
            var VerifyParameterList = await VerifyParameters(ServiceCofig, data);

            if (VerifyParameterList.code != ExecutionState.SUCCESS)
            {
                Logger.LogInformation($"请求事件id:{id},服务名称:{InterfaceServiceName},参数验证失败，失败原因:{VerifyParameterList.message}");
                analyticResult.code = VerifyParameterList.code;
                analyticResult.message = VerifyParameterList.message;
                analyticResult.data = "E";
                return analyticResult;
            }

            List<MESInterfaceParametersConfigDto> parameters = VerifyParameterList.data == null ? new List<MESInterfaceParametersConfigDto>() : VerifyParameterList.data as List<MESInterfaceParametersConfigDto>;

            //判断执行脚本在不在
            var VerificationScriptresult = await VerificationScript(ServiceCofig, id);
            if (VerificationScriptresult.code == ExecutionState.FAIL)
            {
                VerificationScriptresult.data = "E";
                return VerificationScriptresult;
            }

            //校验执行
            if (ServiceCofig.InterfaceType.Equals("校验"))
            {
                var ExecResult = await ExecScrip(ServiceCofig, parameters);

                analyticResult = await AnalyticalResults(ServiceCofig, ExecResult);
            }
            else //数据推送执行
            {
                List<ExecutionResult> analyticResults = new List<ExecutionResult>();

                List<Task> tasks = new List<Task>();

                //parameters.ForEach(p =>
                //{
                //    var tmp = p;
                //    if (tmp.ParameterType.Equals("数组"))
                //    {
                //        tasks.Add(SaveData(InterfaceServiceName, tmp, id, analyticResults));
                //    }
                //});

                //Task.WaitAll(tasks.ToArray());
                for (int i = 0; i < parameters.Count; i++)
                {
                    await SaveData(InterfaceServiceName, parameters[i], id, analyticResults);
                }

                if (analyticResults.Any(ar => ar.code == ExecutionState.FAIL))
                {
                    analyticResult.code = ExecutionState.FAIL;
                    analyticResult.message = "数据保存失败，请联系MES管理员处理。";
                    analyticResult.data = "E";
                    analyticResult.details = analyticResults;
                    var errMSG = "";
                    analyticResults.Where(a => a.code == ExecutionState.FAIL).ToList().ForEach(a =>
                    {
                        errMSG += a.message + ";";
                    });
                    Logger.LogInformation($"请求事件id:{id},服务名称:{InterfaceServiceName},数据保存失败，失败原因:{errMSG}");
                }
                else
                {
                    analyticResult = await AnalyticalResults(ServiceCofig, await ExecScrip(ServiceCofig, id));
                    //analyticResult.code = ExecutionState.SUCCESS;
                    //analyticResult.message = "数据处理成功";
                    //analyticResult.data = "S";
                    Logger.LogInformation($"请求事件id:{id},服务名称:{InterfaceServiceName},数据处理成功,返回信息：{JsonConvert.SerializeObject(analyticResult)}");
                }
            }

            Logger.LogInformation($"当前请求事件[{id}],服务名称:{InterfaceServiceName},执行结果:{analyticResult.code},返回信息：{analyticResult.message}");

            return analyticResult;
        }
        public  Task<ExecutionResult> PutJsonDataSaveAsync(string data, string ConsumerId, HttpContext httpContext, string id = "")
        {
            ExecutionResult analyticResult = new ExecutionResult();
            var InterfaceServiceName = "";

            //如果参数json是带转义字符的先去掉转义字符
            if (data.StartsWith("\"") && data.EndsWith("\""))
            {
                data = data.Replace("\\", "").Remove(0, 1)
                    .Remove(data.Replace("\\", "").Remove(0, 1).Length - 1, 1);
            }

            var Roottoken = JsonHelper.ReadJSON(data);

            if (data.Contains("InterfaceServiceName") || data.Contains("INTERFACESERVICENAME"))
            {
                var jtoken = Roottoken.SelectToken("InterfaceServiceName") == null ? Roottoken.SelectToken("INTERFACESERVICENAME") : Roottoken.SelectToken("InterfaceServiceName");

                //获取服务名称
                InterfaceServiceName = jtoken.ToString();
            }
            else
            {
                analyticResult.code = ExecutionState.FAIL;
                analyticResult.message = "当前请求没有给定服务名称！";
                return Task.FromResult(analyticResult);
            }
          return  PutJsonDataSaveAsync(InterfaceServiceName, data, ConsumerId, httpContext, id);
        }

        /// <summary>
        /// 保存数据
        /// </summary>
        /// <param name="InterfaceServiceName"></param>
        /// <param name="parameter"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        private async Task SaveData(string InterfaceServiceName, MESInterfaceParametersConfigDto parameter, string id, List<ExecutionResult> analyticResults)
        {
            var analyticResult = new ExecutionResult();

            var Pdata = JsonHelper.GetToJsonList(parameter.ParameterValue);
            var tbmodel = await CreateDataTable($"{InterfaceServiceName}_{parameter.ParameterName}", Pdata, id);
            if (tbmodel == null && parameter.CheckDdata)
            {
                analyticResult.code = ExecutionState.FAIL;
                analyticResult.message = $"未获取到参数{parameter.ParameterName}的数据。";
                _logger.LogInformation($"当前请求事件[{id}]，服务名称:{InterfaceServiceName},未获取到参数{parameter.ParameterName}的数据。");
            }
            else if (tbmodel == null && !parameter.CheckDdata)
            {
                analyticResult.code = ExecutionState.SUCCESS;
                analyticResult.message = $"参数{parameter.ParameterName}的数据为空,不需要保存";
                _logger.LogInformation($"当前请求事件[{id}]，服务名称:{InterfaceServiceName},参数{parameter.ParameterName}的数据为空,不需要保存");
            }
            if (tbmodel != null && tbmodel.CreateTableStatus != CreateTableStatus.Fail)
            {
                await _mewConfigService.BulkCopy(tbmodel.Data);
                analyticResult.code = ExecutionState.SUCCESS;
                analyticResult.data = "S";
                _logger.LogInformation($"当前请求事件[{id}]，服务名称:{InterfaceServiceName},表[{tbmodel.TableName}]数据写入成功，共计[{tbmodel.Data.Rows.Count}]条记录");
                analyticResult.message += $"{"\n"}当前请求事件[{id}],表[{tbmodel.TableName}]数据写入成功，共计[{tbmodel.Data.Rows.Count}]条记录";
            }
            analyticResults.Add(analyticResult);
            return;
        }

        /// <summary>
        /// 解析验证脚本结果
        /// </summary>
        /// <param name="serviceConfig"></param>
        /// <param name="ds"></param>
        /// <returns></returns>
        private async Task<ExecutionResult> AnalyticalResults(MESInterfaceServiceConfigInfo serviceConfig, DataSet ds)
        {
            //获取配置的返回信息
            var ResultDataConfigRepository = serviceProvider.GetService<IRepository<MESInterfaceResultDataConfigInfo>>();
            var ResultDataConfig = await ResultDataConfigRepository.SingleOrDefaultAsync(r => r.MESInterfaceServiceConfigInfoID == serviceConfig.MESInterfaceServiceConfigInfoID); //serviceConfig.MesInterfaceResultDataConfigInfos.SingleOrDefault();

            //   var analyticResult = new AnalyticResult();

            if (ds != null || ds.Tables.Count > 0)
            {
                var analyticResultlDT = ds.Tables[0].Copy();
                var analyticResult = analyticResultlDT.ToList<ExecutionResult>().FirstOrDefault();
                if (analyticResult == null)
                {
                    return ExecutionResult.Failed("返回的第一张表不匹配系统返回对象");
                }
                ds.Tables.RemoveAt(0);
                if (ds.Tables.Count > 0 && ds.Tables[0].Rows.Count > 0)
                {
                    IDataSet2JsonService dataSet2Json = new DefaultDataSet2JsonService();
                    var ResultData = dataSet2Json.BuildDictionary(ds);
                    //  int TableInxde = 0;
                    //var ResultData = ds.Tables[TableInxde].Copy().ToList();
                    //if (ds.Tables.Count > 1)
                    //{
                    //    foreach (var key in ResultData[0].Keys)
                    //    {
                    //        if (ResultData[0][key].ToString().Equals("fill"))
                    //        {
                    //            ResultData[0][key] = ds.Tables[++TableInxde].Copy().ToList();
                    //        }
                    //        if (ResultData[0][key].ToString() == "fill:[string]")
                    //        {
                    //            ResultData[0][key] = JsonHelper.ToStringList(ds.Tables[++TableInxde].Copy());
                    //        }
                    //    }
                    //}

                    if (ResultDataConfig.Parameter.Contains("Data"))
                    {
                        if (ResultDataConfig.DataType.Contains("Object"))
                            analyticResult.data = ResultData.SingleOrDefault();
                        if (ResultDataConfig.DataType.Contains("Array"))
                            analyticResult.data = ResultData;
                    }
                }

                return analyticResult;
            }
            else
            {
                return ExecutionResult.Failed("没有获取到执行结果，请联系MES管理员");
                //analyticResult.code = AnalyticState.FAIL;
                //analyticResult.data = "";
                //analyticResult.message = "没有获取到执行结果，请联系MES管理员";
            }
        }

        /// <summary>
        /// 执行脚本
        /// </summary>
        /// <param name="ServiceCofig">服务对象</param>
        /// <param name="parameters">参数集合</param>
        /// <param name="fileIDs">文件ID 以;隔开</param>
        /// <returns>返回执行结果</returns>
        private async Task<DataSet> ExecScrip(MESInterfaceServiceConfigInfo ServiceCofig, List<MESInterfaceParametersConfigDto> parameters, string fileIDs = "")
        {
            List<SqlParameter> sqlParameters = new List<SqlParameter>();

            if (parameters.Count > 0)
            {
                parameters.ForEach(p =>
                {
                    sqlParameters.Add(new SqlParameter(p.ParameterName, p.ParameterValue));
                });
                if (!string.IsNullOrEmpty(fileIDs))
                    sqlParameters.Add(new SqlParameter("@FileIDs", fileIDs));
            }

            return await _mewConfigService.ExecutionScript(ServiceCofig.ScriptCodeName, sqlParameters.ToArray());
        }

        /// <summary>
        /// 执行脚本
        /// </summary>
        /// <param name="ServiceCofig">服务对象</param>
        /// <param name="parameters">参数集合</param>
        /// <returns>返回执行结果</returns>
        private async Task<DataSet> ExecScrip(MESInterfaceServiceConfigInfo ServiceCofig, string id)
        {
            var pp = new Microsoft.Data.SqlClient.SqlParameter[]
            {
                new Microsoft.Data.SqlClient.SqlParameter("@Groupid", id)
            };

            return await _mewConfigService.ExecuteStoredProcedureResultDataSet(ServiceCofig.ScriptCodeName, pp);
        }

        /// <summary>
        /// 验证脚本是否存在
        /// </summary>
        /// <param name="ServiceCofig"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        private async Task<ExecutionResult> VerificationScript(MESInterfaceServiceConfigInfo ServiceCofig, string id)
        {
            ExecutionResult analyticResult = new ExecutionResult();
            var sql = $@"if exists(select * from dbo.sysobjects where id = object_id(N'[dbo].[{ServiceCofig.ScriptCodeName}]') and OBJECTPROPERTY(id, N'IsProcedure') = 1) begin
	                       select 'exists' as ResultMSG
                            end
                            else
                             select 'no exists' as ResultMSG";

            var ds = await _mewConfigService.ExecuteSql(sql);
            if (ds != null && ds.Tables.Count > 0 && ds.Tables[ds.Tables.Count - 1].Rows.Count > 0 && ds.Tables[ds.Tables.Count - 1].Rows[0][0].ToString().ToUpper().Equals("NO EXISTS"))
            {
                analyticResult.code = ExecutionState.FAIL;
                analyticResult.message = $"当前接口服务类型:{ServiceCofig.InterfaceServiceName}，执行的脚本[{ServiceCofig.ScriptCodeName}]不存在";
                _logger.LogError($"当前请求事件[{id}]，当前接口服务类型:{ServiceCofig.InterfaceServiceName}，执行的脚本[{ServiceCofig.ScriptCodeName}]不存在");
            }
            else
                analyticResult.code = ExecutionState.SUCCESS;

            return analyticResult;
        }

        /// <summary>
        /// 验证服务
        /// </summary>
        /// <param name="ServiceName">服务名称</param>
        /// <param name="InterfaceType">服务类型</param>
        /// <param name="id">事件id</param>
        /// <returns></returns>
        public async Task<ExecutionResult> VerifyService(string ConsumerId, string ServiceName)
        {
            var result = new ExecutionResult();

            MESInterfaceConfigInfo mesInterfaceConfigInfo = null;
            if (string.IsNullOrWhiteSpace(ConsumerId))
            {
                mesInterfaceConfigInfo = await _mewConfigService.GetAsync(mc => mc.InterfaceConsumer.Equals("__MES__"));
            }
            else
            {
                mesInterfaceConfigInfo = await _mewConfigService.GetAsync(mc => mc.ConsumerId == ConsumerId);
            }
            if (mesInterfaceConfigInfo == null)
            {
                result.code = ExecutionState.FAIL;
                result.message = $"ConsumerId:{ConsumerId}错误,请检查";
                return result;
            }
            //IRepository<MesInterfaceServiceConfigInfo>
            var InterfaceService = mesInterfaceConfigInfo.MesInterfaceServiceConfigInfos.SingleOrDefault(service => service.InterfaceServiceName == ServiceName);
            if (InterfaceService == null)
            {
                result.code = ExecutionState.FAIL;
                result.message = $"服务名称:{ServiceName}错误,请检查";
                return result;
            }

            result.data = InterfaceService;
            result.details = mesInterfaceConfigInfo;
            result.code = ExecutionState.SUCCESS;
            var mesInterfaceConfig = InterfaceService.MesInterfaceConfig;
            return result;
        }

        /// <summary>
        /// 验证参数并且获取参数值
        /// </summary>
        /// <param name="serviceConfig"></param>
        /// <param name="GetParametValueFun"></param>
        /// <returns></returns>
        public async Task<ExecutionResult> VerifyParameters(MESInterfaceServiceConfigInfo serviceConfig, Func<string, ExecutionResult> GetParametValueFun)
        {
            if (GetParametValueFun == null)
                return ExecutionResult.Failed("未指定获取参数值的函数");
            var result = new ExecutionResult(ExecutionState.SUCCESS, "");

            var parametersConfigDto = new List<MESInterfaceParametersConfigDto>();
            var InterfaceparmetersRepository = serviceProvider.GetService<IRepository<MESInterfaceParametersConfig>>();
            var ParametersConfig = await InterfaceparmetersRepository.GetListAsync(p => p.MESInterfaceServiceConfigInfoID == serviceConfig.MESInterfaceServiceConfigInfoID);//serviceConfig.MesInterfaceParameters.ToList();

            if (ParametersConfig.Count <= 0)
                return ExecutionResult.Success();

            List<Task> taskList = new List<Task>();
            //拿到父级参数
            List<ExecutionResult> resultlist = new List<ExecutionResult>();

            var ParentParameter = ParametersConfig.Where(p => p.Parent).ToList();
            //如果没有父级参数就只有一层参数
            if (ParentParameter != null && ParentParameter.Count > 0)
            {
                ParentParameter.ForEach(pp =>
                {
                    //
                    var parentParamet = pp;
                    var result = GetParametValueFun(pp.ParameterName);
                    if (result.code == ExecutionState.FAIL)//参数不包含父级参数
                    {
                        result.code = ExecutionState.FAIL;
                        result.message += $"参数:{pp.ParameterName}未指定！\n";
                    }
                    else//线程校验子集参数
                    {
                        taskList.Add(Task.Run(() =>
                        {
                            resultlist.Add(VerifyParameterDetail(serviceConfig, parentParamet, result.data?.ToString()));
                        }));
                    }
                });

                await Task.WhenAll(taskList.ToArray());
                var errresult = resultlist.Where(r => r.code == ExecutionState.FAIL)?.ToList();
                if (errresult.Count > 0)
                {
                    result.code = ExecutionState.FAIL;
                    errresult.ForEach(err =>
                    {
                        result.message += err.message + "\n";
                    });
                    return result;
                }
                resultlist.ToList().ForEach(r =>
                {
                    parametersConfigDto.Add(new MESInterfaceParametersConfigDto()
                    {
                        ParameterName = r.data.ToString(),
                        ParameterValue = r.details.ToString()
                    });
                });
            }

            serviceConfig.MesInterfaceParameters.Where(p => string.IsNullOrWhiteSpace(p.ParentParameterID) && !p.Parent).ToList().ForEach(p =>
            {
                var tmpresult = GetParametValueFun(p.ParameterName);
                if (tmpresult.code == ExecutionState.SUCCESS) //参数存在就保存
                {
                    parametersConfigDto.Add(new MESInterfaceParametersConfigDto()
                    {
                        ParameterType = p.ParameterType,
                        ParameterName = p.ParameterName,
                        CheckDdata = p.CheckDdata,
                        ParameterValue = tmpresult.data?.ToString()
                    });
                }

                if (tmpresult.code == ExecutionState.FAIL && p.CheckDdata) //如果参数不存在，并且是需要验证的参数！
                {
                    result.code = ExecutionState.FAIL;
                    result.message += $"参数:{p.ParameterName}未指定！\n";
                }
            });

            result.data = parametersConfigDto;
            return result;
        }

        /// <summary>
        /// 验证参数
        /// </summary>
        /// <param name="serviceConfig"></param>
        /// <param name="Parameters"></param>
        /// <returns></returns>
        public async Task<ExecutionResult> VerifyParameters(MESInterfaceServiceConfigInfo serviceConfig, string Parameters)
        {
            var result = new ExecutionResult(ExecutionState.SUCCESS, "");

            var parametersConfigDto = new List<MESInterfaceParametersConfigDto>();
            var InterfaceparmetersRepository = serviceProvider.GetService<IRepository<MESInterfaceParametersConfig>>();
            var ParametersConfig = await InterfaceparmetersRepository.GetListAsync(p => p.MESInterfaceServiceConfigInfoID == serviceConfig.MESInterfaceServiceConfigInfoID);//serviceConfig.MesInterfaceParameters.ToList();
            if (ParametersConfig.Count <= 0)
                return ExecutionResult.Success();
            List<Task> taskList = new List<Task>();
            //拿到父级参数
            List<ExecutionResult> resultlist = new List<ExecutionResult>();
            var jToken = JsonHelper.ReadJSON(Parameters);
            var ParentParameter = ParametersConfig.Where(p => p.Parent).ToList();
            //如果没有父级参数就只有一层参数
            if (ParentParameter != null && ParentParameter.Count > 0)
            {
                ParentParameter.ForEach(pp =>
                {
                    //
                    var parentParamet = pp;
                    if (!Parameters.Contains(pp.ParameterName))//参数不包含父级参数
                    {
                        result.code = ExecutionState.FAIL;
                        result.message += $"参数:{pp.ParameterName}未指定！\n";
                    }
                    else//线程校验子集参数
                    {
                        taskList.Add(Task.Run(() =>
                        {
                            resultlist.Add(VerifyParameterDetail(serviceConfig, parentParamet, jToken.SelectToken(pp.ParameterName).ToString()));
                        }));
                    }
                });

                await Task.WhenAll(taskList.ToArray());
                var errresult = resultlist.Where(r => r.code == ExecutionState.FAIL)?.ToList();
                if (errresult.Count > 0)
                {
                    result.code = ExecutionState.FAIL;
                    errresult.ForEach(err =>
                    {
                        result.message += err.message + "\n";
                    });
                    return result;
                }
                resultlist.ToList().ForEach(r =>
                {
                    parametersConfigDto.Add(new MESInterfaceParametersConfigDto()
                    {
                        ParameterName = r.data.ToString(),
                        ParameterValue = r.details.ToString()
                    });
                });
            }

            serviceConfig.MesInterfaceParameters.Where(p => string.IsNullOrWhiteSpace(p.ParentParameterID) && !p.Parent).ToList().ForEach(p =>
            {
                if (Parameters.Contains(p.ParameterName)) //参数存在就保存
                {
                    parametersConfigDto.Add(new MESInterfaceParametersConfigDto()
                    {
                        ParameterType = p.ParameterType,
                        ParameterName = p.ParameterName,
                        CheckDdata = p.CheckDdata,
                        ParameterValue = jToken.SelectToken(p.ParameterName)?.ToString()
                    });
                }

                if (!Parameters.Contains(p.ParameterName) && p.CheckDdata) //如果参数不存在，并且是需要验证的参数！
                {
                    result.code = ExecutionState.FAIL;
                    result.message += $"参数:{p.ParameterName}未指定！\n";
                }
            });

            result.data = parametersConfigDto;
            return result;
        }

        /// <summary>
        /// 验证父级参数下面的所有子参数
        /// </summary>
        /// <param name="serviceConfig"></param>
        /// <param name="ParentParametersConfig"></param>
        /// <param name="ParentParameters"></param>
        /// <returns></returns>
        private ExecutionResult VerifyParameterDetail(MESInterfaceServiceConfigInfo serviceConfig, MESInterfaceParametersConfig ParentParametersConfig, string ParentParameters)
        {
            var result = new ExecutionResult(ExecutionState.SUCCESS, "");

            var ParametList = serviceConfig.MesInterfaceParameters.Where(p =>
                  p.ParentParameterID == ParentParametersConfig.MESInterfaceParametersConfigID).ToList();

            ParametList.ForEach(p =>
            {
                if (!ParentParameters.Contains(p.ParameterName) && p.CheckDdata)
                {
                    result.code = ExecutionState.FAIL;
                    result.message += $"参数:{ParentParametersConfig.ParameterName}对象为{ParentParametersConfig.ParameterType},不包含子参数:{p.ParameterName}\n";
                }
            });

            if (result.code == ExecutionState.SUCCESS)
            {
                result.data = ParentParametersConfig.ParameterName;
                result.details = ParentParameters;
            }

            return result;
        }

        private async Task<TableModel> CreateDataTable(string TableName, JArray jlist, string id = "")
        {
            if (jlist.Count <= 0)
                return null;
            var model = CreateTableHelper.BuildTableModel(TableName, jlist, id);

            var ds = await _mewConfigService.ExecuteSql(CreateTableHelper.CheckTableExists);
            if (ds != null && ds.Tables.Count > 0 && ds.Tables[ds.Tables.Count - 1].Rows.Count > 0 && ds.Tables[ds.Tables.Count - 1].Rows[0][0].ToString().ToUpper().Equals("N"))
            {
                var sql = CreateTableHelper.BulidCreateTableSQL(model);
                _logger.LogInformation($"正在创建表[{TableName}],sql;{sql}");
                await _mewConfigService.ExecuteSql(sql);

                model.CreateTableStatus = CreateTableStatus.Success;
            }
            else if (ds != null && ds.Tables.Count > 0 && ds.Tables[ds.Tables.Count - 1].Rows.Count > 0 &&
                     ds.Tables[ds.Tables.Count - 1].Rows[0][0].ToString().ToUpper().Equals("Y"))
            {
                model.CreateTableStatus = CreateTableStatus.Exist;
                var dbinfo = await _mewConfigService.QueryDBTableInfo(CreateTableHelper.QueryDBTableInfo);
                CreateTableHelper.TableDiff(model, dbinfo.ToList());
                if (model.AlterColumnDescription.Count > 0)
                {
                    var sql = CreateTableHelper.BulidNewFieldSQL(model);
                    await _mewConfigService.ExecuteSql(sql);
                }
            }

            return model;
        }
    }
}