﻿using OrBit.MESInterface.Application.Contracts.WGTask;
using OrBit.MESInterface.Application.Contracts.WGTask.Dtos;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using Volo.Abp.Application.Services;

namespace OrBit.MESInterface.Application.Service.WGTask
{
    public class GenerateTubeSetDataService : ApplicationService, IGenerateTubeSetDataService
    {
        private TubeSetDataInfoDto TubeSetData { get; set; }

        private XmlDocument doc { get; set; }
        public GenerateTubeSetDataService()
        {
            doc = new XmlDocument();
            //添加文档声明
            var declaration = doc.CreateXmlDeclaration("1.0", "", "");
            doc.AppendChild(declaration);
        }
        public XmlDocument Build(TubeSetDataInfoDto tubeSetData)
        {
            this.TubeSetData = tubeSetData;
            //添加根节点
            var root = CreateRoot();
            //添加info
            root.AppendChild(CreateInfo(tubeSetData.Name, tubeSetData.Time));

            //添加tube
            var tubeset = CreateTubeSet(tubeSetData.Tubes, tubeSetData.Waste);
            root.AppendChild(tubeset);
            return doc;

        }

        public void SaveFile(string fileName)
        {
            doc.Save(fileName);
        }

        ///添加根节点
        XmlElement CreateRoot()
        {
            var root = doc.CreateElement("Project");
            root.SetAttribute("ID", "80204");
            root.SetAttribute("Version", "4.0");
            doc.AppendChild(root);
            return root;
        }

        ///创建info信息
        XmlElement CreateInfo(string name, string dateTime, string software = "tControl")
        {
            var info = doc.CreateElement("Info");
            var lastChangedBy = doc.CreateElement("LastChangedBy");
            lastChangedBy.SetAttribute("Name", name);
            lastChangedBy.SetAttribute("DateTime", dateTime);
            lastChangedBy.SetAttribute("Software", software);
            info.AppendChild(lastChangedBy);
            return info;
        }

        /// <summary>
        /// 创建TubeSet
        /// </summary>
        /// <param name="tubeList">组合切管信息</param>
        /// <param name="waste">是否有报废信息</param>
        /// <returns></returns>
        XmlElement CreateTubeSet(List<TubeInfoDto> tubeList, bool waste = true)
        {
            var general = CreateGeneral(TubeSetData.LengthInLoading, TubeSetData.TimeBlowOut, TubeSetData.NumberOfTubeSets);
            var tubeSet = doc.CreateElement("TubeSet");

            tubeSet.AppendChild(general);
            //固定切除100为废料
            if (waste)
            {
                var tube = doc.CreateElement("Tube");
                tube.SetAttribute("No", "1");
                tube.SetAttribute("Length", "100");
                tube.SetAttribute("Pieces", "1");
                tube.SetAttribute("Storage", "-1");
                tube.SetAttribute("Support", "0");
                tube.SetAttribute("Name", "");
                tubeSet.AppendChild(tube);
            }

            if (tubeList != null && tubeList.Count > 0)
            {
                tubeList.ForEach(tb =>
                {
                    var tube = doc.CreateElement("Tube");
                    tube.SetAttribute("No", (int.Parse(tb.no) + 1).ToString());
                    tube.SetAttribute("Length", tb.Length);
                    tube.SetAttribute("Pieces", tb.Pieces);
                    tube.SetAttribute("Storage", tb.Storage);
                    tube.SetAttribute("Support", tb.Support);
                    tube.SetAttribute("Name", tb.ProductName);
                    tubeSet.AppendChild(tube);
                });
                for (int i = waste ? tubeList.Count() + 1 : tubeList.Count(); i < 10; i++)
                {
                    //tubeList.ForEach(tb =>
                    //{
                    var tube = doc.CreateElement("Tube");
                    tube.SetAttribute("No", (i + 1).ToString());
                    tube.SetAttribute("Length", "0");
                    tube.SetAttribute("Pieces", "0");
                    tube.SetAttribute("Storage", "0");
                    tube.SetAttribute("Support", "0");
                    tube.SetAttribute("Name", "");
                    tubeSet.AppendChild(tube);
                    //});
                }
            }

            tubeSet.AppendChild(CreateRest());
            return tubeSet;
        }

        ///创建Rest
        XmlElement CreateRest()
        {
            var rest = doc.CreateElement("Rest");
            rest.SetAttribute("Storage", "-1");

            return rest;
        }

        ///创建General
        XmlElement CreateGeneral(string lengthInLoading, string timeBlowOut, string numberOfTubeSets, string optimizedUnloading = "0")
        {
            var general = doc.CreateElement("General");

            //原材料管子长度
            general.SetAttribute("LengthInLoading", lengthInLoading);
            //吹气量（程序中来）
            general.SetAttribute("TimeBlowOut", timeBlowOut);
            //原材料管子个数
            general.SetAttribute("NumberOfTubeSets", numberOfTubeSets);
            //0(固定值)
            general.SetAttribute("OptimizedUnloading", optimizedUnloading);
            return general;
        }
    }
}
