﻿using Microsoft.Data.SqlClient;
using OrBit.MESInterface.Domain.File.Entities;
using OrBit.MESInterface.Domain.File.Repository;
using System.Threading.Tasks;
using Volo.Abp.Domain.Services;
using Volo.Abp.Threading;

namespace OrBit.MESInterface.Domain.File.Service
{
    public class APIFileManager : DomainService
    {
        public readonly IAPIFileRepository _repository;

        public APIFileManager(IAPIFileRepository repository)
        {
            _repository = repository;
        }

        public async Task<string> GetUserIDByUserName(string userName)
        {
            var ds =  await _repository.ExecuteSql("EXEC MESInterfaceCheckUser @userName=@tmpuserName ", new SqlParameter[] {
                new SqlParameter("@tmpuserName",userName)
            });
            if (ds != null && ds.Tables.Count > 0 && ds.Tables[0].Rows.Count > 0)
            {
                return ds.Tables[0].Rows[0]["UserId"].ToString();
            }
            return string.Empty;
        }

        public  async Task<string> GetPKID()
        {
            var ds = await _repository.ExecuteSql($@"DECLARE @PKID CHAR(12);
                                        EXEC dbo.SysGetObjectPKId @ObjectName = N'APIFILE',
                                        @PKID = @PKID OUTPUT
                                        SELECT @PKID AS pkid");

            if (ds != null && ds.Tables.Count > 0 && ds.Tables[0].Rows.Count > 0)
            {
                return ds.Tables[0].Rows[0]["pkid"].ToString();
            }
            return string.Empty;
        }

        /// <summary>
        /// 根据hashid查询文件
        /// </summary>
        /// <param name="hashid"></param>
        /// <returns></returns>
        public async Task<APIFile> QueryFileInfoByHASH(string hashid, string extensionName)
        {
            return await _repository.FindAsync(file => file.HashID.Equals(hashid) && file.ExtensionName.Equals(extensionName));
        }
    }
}