﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.Data.SqlClient;
using OrBit.MESInterface.Domain.RemontQuery.Repository;
using OrBit.MESInterface.Infrastructure;
using OrBit.MESInterface.Infrastructure.Extensions;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Volo.Abp.Domain.Services;
using static System.Runtime.InteropServices.JavaScript.JSType;

namespace OrBit.MESInterface.Domain.RemontQuery.Service
{
    public class RemontQueryManager : DomainService
    {
        public readonly IRemontQuerylogsRepository RemontQuerylogsRepository;
        public RemontQueryManager(IRemontQuerylogsRepository RemontQuerylogsRepository) { 
            this.RemontQuerylogsRepository= RemontQuerylogsRepository;
        }

        private string SqlParameterToString(SqlParameter[] sqlParameters = null)
        {
            if (sqlParameters == null || sqlParameters.Length <= 0)
                return "";
            var sb = new StringBuilder();
         
            sb.AppendLine($"Parameter:");
            for (int i = 0; i < sqlParameters.Length; i++)
            {
                sb.AppendLine($"{sqlParameters[i].ParameterName}:{sqlParameters[i].Value},");

            }
          
            sb.AppendLine(";");

            return sb.ToString();
        }
        private Task<DataSet> QueryData(string commandStr, string commandType, SqlParameter[] sqlParameters = null)
        {
            if (commandType.Equals("SQL"))
            {
                return RemontQuerylogsRepository.ExecuteSql(commandStr, sqlParameters);
            }
            else
            {
                return RemontQuerylogsRepository.ExecuteStoredProcedureResultDataSet(commandStr, sqlParameters);
            }
        }
        public async Task<ExecutionResult> QueryData(string userName, string ip, string commandStr, string commandType, SqlParameter[] sqlParameters = null)
        {
            if (string.IsNullOrWhiteSpace(commandStr))
                return ExecutionResult.Failed("查询字符串不能为空");
            if (string.IsNullOrWhiteSpace(commandType))
                return ExecutionResult.Failed("查询字符串类型不能为空");

            var logs = await RemontQuerylogsRepository.InsertAsync(new Domain.RemontQuery.Entities.RemontQuerylogs
            {
                CommandStr = commandStr,
                CommandType = commandType,
                Parameters = SqlParameterToString(sqlParameters),
                QueryTime = DateTime.Now,
                QueryUserName = userName,
                ExecMsg = "开始执行，还未返回",
                RemontQuerylogsID = Guid.NewGuid().ToString(),
                Ip = ip
            }, true);

            var queryResult = await QueryData(commandStr, commandType, sqlParameters).ContinueWith(t =>
            {
                if (t.IsFaulted)
                {
                    logs.ExecResult = false;
                    logs.ExecMsg = t.Exception.InnerException.Message;
                    return ExecutionResult.Failed(t.Exception.InnerException.Message);
                }
                else
                {
                    logs.ExecResult = true;
                    logs.ExecMsg = "查询成功";
                    return ExecutionResult.Success("查询成功", t.Result);
                }
            });

            await RemontQuerylogsRepository.UpdateAsync(logs, true);

            if (queryResult.code != ExecutionState.SUCCESS)
            {
                return queryResult;
            }
            else
            {
                Dictionary<string, List<Dictionary<string, object>>> ds = new Dictionary<string, List<Dictionary<string, object>>>();
                for (int i = 0; i < (queryResult.data as DataSet).Tables.Count; i++)
                {
                    DataTable item = (queryResult.data as DataSet).Tables[i];
                    ds.Add("result" + (i + 1), item.ToList());
                }
                queryResult.data = ds;
                return queryResult;
            }
        }
    }
}
