﻿using Microsoft.Data.SqlClient;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Internal;
using OrBit.MESInterface.Infrastructure.CreateTable;
using OrBit.MESInterface.Infrastructure.Extensions;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Volo.Abp.Domain.Entities;
using Volo.Abp.Domain.Repositories.EntityFrameworkCore;
using Volo.Abp.EntityFrameworkCore;
using Volo.Abp.Threading;

namespace OrBit.MESInterface.EntityFrameworkCore.Extensions
{
    public class ExetenEfCoreRepository<TDbContext, TEntity> : EfCoreRepository<TDbContext, TEntity> where TDbContext : IEfCoreDbContext
    where TEntity : class, IEntity
    {

      
        public ExetenEfCoreRepository(IDbContextProvider<TDbContext> dbContextProvider) : base(dbContextProvider)
        {
            
        }

        public virtual  Task<DataSet> ExecutionScript(string script, SqlParameter[] sqlParameters = default)
        {
            var db = AsyncHelper.RunSync(GetDbContextAsync);
            return Task.FromResult(db.ExecuteStoredProcedureResultDataSet(script, sqlParameters));
        }

        public virtual Task BulkCopy<T>(IEnumerable<T> list)
        {
            var dt = list.ToDataTable<T>();
           return  BulkCopy(dt);
        }

        public virtual Task BulkCopy(DataTable dt)
        {
            var db = AsyncHelper.RunSync(GetDbContextAsync);

            using (var bulkCopy = new SqlBulkCopy(db.Database.GetDbConnection().ConnectionString))
            {
                bulkCopy.DestinationTableName = dt.TableName;
                for (var i = 0; i < dt.Columns.Count; i++)
                {
                    bulkCopy.ColumnMappings.Add(dt.Columns[i].ColumnName, dt.Columns[i].ColumnName);
                }
                bulkCopy.WriteToServer(dt);
            }
            return Task.CompletedTask;
        }

        public virtual Task CreateTable(string sql)
        {
            var db = AsyncHelper.RunSync(GetDbContextAsync);
           
           return Task.FromResult(db.Database.ExecuteSqlRawAsync(sql));
           
        }

        public virtual  async Task<IEnumerable<DBTableInfo>> QueryDBTableInfo(string sql, SqlParameter[] parameters = null)
        {
            var db =await GetDbContextAsync();
            return await db.ExecuteSqlResultEnumerable<DBTableInfo>(sql, parameters);
           
           
        }
        public virtual  async Task<DataSet> ExecuteSql(string sql, SqlParameter[] parameters = null)
        {
            var db =await GetDbContextAsync();

            return db.ExecuteSqlResultDataSet(sql, parameters);
            
        }
        public virtual  Task<int> ExecuteStoredProcedureNonQueryAsync(string StoredProcedure, SqlParameter[] parameters = null)
        {
            var db = AsyncHelper.RunSync(GetDbContextAsync);
            
           return Task.FromResult(db.NonQuery(CommandType.StoredProcedure, StoredProcedure, parameters));
          
        }
        public virtual Task<int> ExecuteSqlNonQueryAsync(string StoredProcedure, SqlParameter[] parameters = null)
        {
            var db = AsyncHelper.RunSync(GetDbContextAsync);
           
            return Task.FromResult(db.NonQuery(CommandType.Text, StoredProcedure, parameters));
           
        }

        public virtual async Task<List<T>> ExecuteSqlResultList<T>(string sql, SqlParameter[] parameters = null) where T : class, new()
        {
            var db = AsyncHelper.RunSync(GetDbContextAsync);
            return (await db.ExecuteSqlResultEnumerable<T>(sql, parameters))?.ToList();
        }
        public virtual async Task<T> ExecuteSqlResultT<T>(string sql, SqlParameter[] parameters = null) where T : class, new()
        {
            var db = AsyncHelper.RunSync(GetDbContextAsync);
            var result=await  db.ExecuteSqlResultEnumerable<T>(sql, parameters);
            return result.FirstOrDefault();
        }

        public virtual  Task<DataSet> ExecuteStoredProcedureResultDataSet(string StoredProcedure, SqlParameter[] parameters = null)
        {
            var db = AsyncHelper.RunSync(GetDbContextAsync);
           
                return Task.FromResult(db.ExecuteStoredProcedureResultDataSet(StoredProcedure, parameters));
           
        }
     
        public virtual  Task<DataTable> ExecuteSqlResultDataTable(string StoredProcedure, SqlParameter[] parameters = null)
        {
            var db = AsyncHelper.RunSync(GetDbContextAsync);
            return db.ExecuteSqlResultDataTable(StoredProcedure, parameters);
           
        }
    }
}