﻿using Microsoft.Data.SqlClient;
using OrBit.MESInterface.Domain.MaterialGroup.Entities;
using OrBit.MESInterface.Domain.MaterialGroup.Repository;
using OrBit.MESInterface.Domain.MesInterfaceConfiguration.Entities;
using OrBit.MESInterface.EntityFrameworkCore.Extensions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Volo.Abp.EntityFrameworkCore;
using Volo.Abp.Threading;

namespace OrBit.MESInterface.EntityFrameworkCore.Repositories.MaterialGroup
{
    public class MaterialGroupRepository : ExetenEfCoreRepository<MesInterfaceDBContext, MaterialGroupInfo>, IMaterialGroupRepository
    {
        public MesInterfaceDBContext Context { get;set; }
        

        public MaterialGroupRepository(IDbContextProvider<MesInterfaceDBContext> dbContextProvider) : base(dbContextProvider)
        {
           
        }

        public async Task<List<MaterialGroupInfo>> GetMaterialGroupInfoID(string moTaskName)
        {
            Context = AsyncHelper.RunSync(GetDbContextAsync);
            var materialGroupInfoID = Context.MaterialGroupInfoTaskListInfo.Where(mgi => mgi.MotaskName.Equals(moTaskName)).Select(m => m.MaterialGroupInfoID);
            return await GetListAsync(m => m.MaterialGroupInfoID.Equals(materialGroupInfoID));
        }

       public async Task<IEnumerable<ProcessStepProgramInfo>> QueryProgramListByProductNameAndAssetNumber(string productName, string AssetNumber)
        {
            productName= productName.Replace('.', '_');
            Context = AsyncHelper.RunSync(GetDbContextAsync);
            return await ExecuteSqlResultList<ProcessStepProgramInfo>(@$"SELECT pr.ProductName,psp.ProgramName,psp.Description,ap.FileName,ap.FileUrl,psp.CreateDate
                         FROM dbo.ProcessStepProgramRoot pr
                         JOIN dbo.ProcessStepProgram psp ON psp.ProcessStepProgramRootID = pr.ProcessStepProgramRootID
                         JOIN dbo.APIFile ap ON ap.APIFileID=psp.FileID
						 JOIN dbo.sys_EqAssetInfo e ON e.EAM_EqAssetId=pr.EAM_EqAssetID
                         WHERE e.AssetNumber=@AssetNumber and pr.ProductName=@ProductName AND psp.Activate=1 AND psp.ProcessStepProgramStates=1", [new SqlParameter("@AssetNumber", AssetNumber), new SqlParameter("@ProductName", productName)]);
        }
        public override async Task<IQueryable<MaterialGroupInfo>> WithDetailsAsync()
        {

            return (await GetQueryableAsync()).IncludeDetails();
        }
    }
}
