﻿using Microsoft.AspNetCore.Mvc.Filters;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Logging.Abstractions;
using System;
using System.Text;
using System.Text.RegularExpressions;
using Volo.Abp;
using Volo.Abp.AspNetCore.Mvc;

namespace OrBit.MESInterface.HttpApi.Host.Base.Attribute
{
    [AttributeUsage(AttributeTargets.Class | AttributeTargets.Method, AllowMultiple = false)]
    public class BasicAuthAttribute : ActionFilterAttribute
    {
        /// <summary>
        /// 在控制器执行之前调用
        /// </summary>
        /// <param name="context">执行的上下文</param>
        public override void OnActionExecuting(ActionExecutingContext context)
        {
            var logger = context.GetService<ILogger<BasicAuthAttribute>>(NullLogger<BasicAuthAttribute>.Instance);
            // 判断是否加上了不需要拦截
            string authHeader = context.HttpContext.Request.Headers["Authorization"];

            //获取IP
            var ip = context.HttpContext.Connection.RemoteIpAddress;//Current.Request.UserHostAddress;

            if (authHeader != null && authHeader.StartsWith("Basic"))
            {
                //Extract credentials
                string encodedUsernamePassword = authHeader.Substring("Basic ".Length).Trim();
                Encoding encoding = Encoding.GetEncoding("iso-8859-1");
                string usernamePassword = encoding.GetString(Convert.FromBase64String(encodedUsernamePassword));
                int seperatorIndex = usernamePassword.IndexOf(':');
                var username = usernamePassword.Substring(0, seperatorIndex);
                var password = usernamePassword.Substring(seperatorIndex + 1);

                var ActionName = ((Microsoft.AspNetCore.Mvc.Controllers.ControllerActionDescriptor)context.ActionDescriptor).ActionName;

                if (!Regex.IsMatch(username, @"^[A-Za-z0-9_]+$"))
                {
                    logger.LogInformation("Wrong user name format");
                    throw new UserFriendlyException($"Wrong user name format", "-1", logLevel: LogLevel.Error);
                }

                //if (IsAuthorized(username, password))
                if (ValidateUser(username, password))
                {
                    logger.LogInformation($"Authorized, user {username}");

                    return;
                }
                else
                {
                    logger.LogInformation($"Unauthorized, user {username} ");
                    throw new UserFriendlyException($"Unauthorized, user {username}", "-1", logLevel: LogLevel.Error);
                }
            }
            else
            {
                // var ex= new Exception("No Basic Auth Info");
                logger.LogInformation("No Basic Auth Info");
                //  var exceptionHandlingOptions = context.GetRequiredService<IOptions<AbpExceptionHandlingOptions>>().Value;
                //  var exceptionToErrorInfoConverter = context.GetRequiredService<IExceptionToErrorInfoConverter>();
                //  var remoteServiceErrorInfo = exceptionToErrorInfoConverter.Convert(ex, exceptionHandlingOptions.SendExceptionsDetailsToClients);

                //  remoteServiceErrorInfo.Code = "-1";
                //  remoteServiceErrorInfo.Message = "No Basic Auth Info";
                //  logger.LogException(ex, LogLevel.Error);
                //  context.Result = new ObjectResult(remoteServiceErrorInfo);
                throw new UserFriendlyException($"No Basic Auth Info", "-1", logLevel: LogLevel.Error);
            }
        }

        public bool ValidateUser(string accountName, string password)
        {
            //验证用户名密码

            return "sapuser_dev".Equals(accountName, StringComparison.InvariantCultureIgnoreCase)
                   && "!@#456&*".Equals(password); ;
        }
    }

    /// <summary>
    /// 不需要登陆的地方加个这个空的拦截器
    /// </summary>
    public class NoSignAttribute : ActionFilterAttribute
    { }
}