﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Abstractions;
using Microsoft.AspNetCore.Mvc.Filters;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Logging.Abstractions;
using Microsoft.Extensions.Options;
using System;
using System.Text;
using System.Threading.Tasks;
using Volo.Abp;
using Volo.Abp.AspNetCore.ExceptionHandling;
using Volo.Abp.AspNetCore.Mvc;
using Volo.Abp.AspNetCore.Mvc.ExceptionHandling;
using Volo.Abp.Authorization;
using Volo.Abp.Domain.Entities;
using Volo.Abp.ExceptionHandling;
using Volo.Abp.Http;
using Volo.Abp.Json;
using Volo.Abp.Validation;

namespace OrBit.MESInterface.HttpApi.Host.Base.Filters
{
    //public class GlobalExceptionFilter : AbpExceptionFilter
    //{
    //    private readonly ILogger<GlobalExceptionFilter> logger;

    //    public GlobalExceptionFilter(ILogger<GlobalExceptionFilter> logger)
    //    {
    //        this.logger = logger;
    //    }

    //    protected override async Task HandleAndWrapException(ExceptionContext context)
    //    {
    //        var exceptionHandlingOptions = context.GetRequiredService<IOptions<AbpExceptionHandlingOptions>>().Value;
    //        var exceptionToErrorInfoConverter = context.GetRequiredService<IExceptionToErrorInfoConverter>();
    //        var remoteServiceErrorInfo = exceptionToErrorInfoConverter.Convert(context.Exception, exceptionHandlingOptions.SendExceptionsDetailsToClients);

    //        var logLevel = context.Exception.GetLogLevel();

    //        var remoteServiceErrorInfoBuilder = new StringBuilder();
    //        remoteServiceErrorInfoBuilder.AppendLine($"---------- {nameof(RemoteServiceErrorInfo)} ----------");
    //        remoteServiceErrorInfoBuilder.AppendLine(context.GetRequiredService<IJsonSerializer>().Serialize(remoteServiceErrorInfo, indented: true));

    //        var logger = context.GetService<ILogger<AbpExceptionFilter>>(NullLogger<AbpExceptionFilter>.Instance);

    //        logger.LogWithLevel(logLevel, remoteServiceErrorInfoBuilder.ToString());

    //        logger.LogException(context.Exception, logLevel);

    //        await context.GetRequiredService<IExceptionNotifier>().NotifyAsync(new ExceptionNotificationContext(context.Exception));

    //        if (context.Exception is AbpAuthorizationException)
    //        {
    //            await context.HttpContext.RequestServices.GetRequiredService<IAbpAuthorizationExceptionHandler>()
    //                .HandleAsync(context.Exception.As<AbpAuthorizationException>(), context.HttpContext);
    //        }
    //        else
    //        {
    //            context.HttpContext.Response.Headers.Add(AbpHttpConsts.AbpErrorFormat, "true");
    //            context.HttpContext.Response.StatusCode = (int)context
    //                .GetRequiredService<IHttpExceptionStatusCodeFinder>()
    //                .GetStatusCode(context.HttpContext, context.Exception);

    //            context.Result = new ObjectResult(new RemoteServiceErrorResponse(remoteServiceErrorInfo));
    //        }

    //        context.Exception = null; //Handled!
    //    }
    //    //public void OnException(ExceptionContext context)
    //    //{
    //    //    logger.LogError(new EventId(context.Exception.HResult),
    //    //   context.Exception,
    //    //   context.Exception.Message);
    //    //    if (context.Exception is AbpValidationException)
    //    //    {
    //    //        var validateerros = ((AbpValidationException)context.Exception).ValidationErrors;
    //    //        context.Result = new JsonResult(new ResultModel <List<System.ComponentModel.DataAnnotations.ValidationResult>>(100, validateerros.Count > 0 ? validateerros[0].ErrorMessage : context.Exception.Message, validateerros.ToList()));
    //    //    }
    //    //    else
    //    //    {
    //    //        var errmsg = ($"url:{context.HttpContext.Request.Path + ";异常:" + context.Exception.Message}");
    //    //        context.Result = new JsonResult(new ResultModel<string>(-1, errmsg,""));
    //    //    }
    //    //    context.ExceptionHandled = true;
    //    //}
    //}

    public class GlobalExceptionFilter : AbpExceptionFilter
    {
        protected override bool ShouldHandleException(ExceptionContext context)
        {
            if (context.ActionDescriptor.IsControllerAction()
                    // &&
                    // context.ActionDescriptor.HasObjectResult()
                    )
            {
                return true;
            }

            if (context.HttpContext.Request.CanAccept(MimeTypes.Application.Json))
            {
                return true;
            }

            if (context.HttpContext.Request.IsAjax())
            {
                return true;
            }

            return false;
        }

        protected override async Task HandleAndWrapException(ExceptionContext context)
        {
            var exceptionHandlingOptions = context.GetRequiredService<IOptions<AbpExceptionHandlingOptions>>().Value;
            var exceptionToErrorInfoConverter = context.GetRequiredService<IExceptionToErrorInfoConverter>();
            var remoteServiceErrorInfo = exceptionToErrorInfoConverter.Convert(context.Exception, exceptionHandlingOptions.SendExceptionsDetailsToClients);
            var logger = context.GetService<ILogger<AbpExceptionFilter>>(NullLogger<AbpExceptionFilter>.Instance);

            context.HttpContext.Response.Headers.Add(AbpHttpConsts.AbpErrorFormat, "true");
            context.HttpContext.Response.StatusCode = 200;// (int)_statusCodeFinder.GetStatusCode(context.HttpContext, context.Exception);

            //  var remoteServiceErrorInfo = _errorInfoConverter.Convert(context.Exception, _exceptionHandlingOptions.SendExceptionsDetailsToClients);
            remoteServiceErrorInfo.Code = "-1";
            remoteServiceErrorInfo.Message = SimplifyMessage(context.Exception);
            // HttpResponse
            context.Result = new ObjectResult(remoteServiceErrorInfo);

            // 写日志
            var logLevel = context.Exception.GetLogLevel();
            var remoteServiceErrorInfoBuilder = new StringBuilder();
            remoteServiceErrorInfoBuilder.AppendLine($"---------- {nameof(RemoteServiceErrorInfo)} ----------");
            remoteServiceErrorInfoBuilder.AppendLine(context.GetRequiredService<IJsonSerializer>().Serialize(remoteServiceErrorInfo, indented: true));
            logger.LogWithLevel(logLevel, remoteServiceErrorInfoBuilder.ToString());
            logger.LogException(context.Exception, logLevel);

            await context.HttpContext
                .RequestServices
                .GetRequiredService<IExceptionNotifier>()
                .NotifyAsync(
                    new ExceptionNotificationContext(context.Exception)
                );

            context.Exception = null; //Handled!
        }

        protected string SimplifyMessage(Exception error)
        {
            string message = string.Empty;
            switch (error)
            {
                case AbpAuthorizationException e:
                    return message = "Authenticate failure！";

                case AbpValidationException e:
                    return message = "Request param validate failure！";

                case EntityNotFoundException e:
                    return message = "not found the entity！";

                case BusinessException e:
                    return message = $"{e.Message}";

                case NotImplementedException e:
                    return message = "not implement！";

                default:
                    return message = error.Message;
            }
        }
    }
}