﻿using Microsoft.AspNetCore.Http;
using OrBit.MESInterface.Application.Contracts.MESInterface;
using OrBit.MESInterface.Infrastructure;
using System;
using System.IO;
using System.Threading.Tasks;

namespace OrBit.MESInterface.HttpApi.Host.Base.Middleware
{
    public class AuthorizationMiddleware
    {
        private readonly RequestDelegate _next;

        private readonly IOrBitAuthorizationService authorizationService;

        public AuthorizationMiddleware(RequestDelegate next, IOrBitAuthorizationService _AuthorizationService)
        {
            _next = next;

            authorizationService = _AuthorizationService;
        }

        /// <summary>
        /// 时间戳Timestamp转换成日期
        /// </summary>
        /// <param name="timeStamp"></param>
        /// <returns></returns>

        public async Task InvokeAsync(HttpContext context)
        {
            if (context.Request.Path.Value.Equals("/api/OrBitMesInterface/PutData"))
            {
                var RequestBody = "";
                context.Request.EnableBuffering();
                var reader = new StreamReader(context.Request.Body);
                RequestBody = await reader.ReadToEndAsync();

                context.Request.Body.Seek(0, SeekOrigin.Begin);
                //拿参数
                var access_token = "";
                var Timestamp = "";
                if (context.Request.Headers.ContainsKey("access_token") && context.Request.Headers.ContainsKey("MESApi-Timestamp"))
                {
                    access_token = context.Request.Headers["access_token"];
                    Timestamp = context.Request.Headers["MESApi-Timestamp"];

                    if ((DateTime.Now.ToUniversalTime().Ticks - 621355968000000000) / 10000000 - int.Parse(Timestamp) > 600)
                    {
                        await context.Response.WriteAsJsonAsync(new ExecutionResult(ExecutionState.FAIL, "时间戳与当前时间相差大于10分钟。请求无效"), typeof(ExecutionResult));

                        return;
                    }
                }
                //拿token

                var Authorization = context.Request.Headers["Authorization"];

                string ConsumerId = context.Request.Headers["ConsumerId"];

                var result = await authorizationService.Authorization(access_token, Timestamp, RequestBody, ConsumerId, Authorization);
                if (result.code != ExecutionState.SUCCESS)
                {
                    context.Response.StatusCode = 401;
                    await context.Response.WriteAsJsonAsync(result, typeof(ExecutionResult));
                    return;
                }
            }

            await _next(context);
        }
    }
}