﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using OrBit.MESInterface.Application.Contracts.ApiFile;
using OrBit.MESInterface.Application.Contracts.ApiFile.Dtos;
using OrBit.MESInterface.Application.Contracts.AppUpdata;
using OrBit.MESInterface.Infrastructure;
using System;
using System.ComponentModel.DataAnnotations;
using System.IO;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Http;
using Volo.Abp.AspNetCore.Mvc;

namespace OrBit.MESInterface.HttpApi.Host.Controllers
{
    /// <summary>
    /// OrBitMes文件上传服务
    /// </summary>
    [ApiController]
    [Route("api/OrBitFileService/[action]")]
    public class OrBitFileServiceControllers : AbpController
    {
        private readonly ILogger<OrBitFileServiceControllers> _logger;

        private readonly IAPIFileAppService _UploadfileServer;

        private readonly IAppUpdataServer _appUpdataServer;

        public OrBitFileServiceControllers(IAPIFileAppService _UploadfileServer,IAppUpdataServer appUpdataServer, ILogger<OrBitFileServiceControllers> _logger
            )
        {
            this._UploadfileServer = _UploadfileServer;
            this._logger = _logger;
            this._appUpdataServer = appUpdataServer;
        }
        /// <summary>
        /// 获取驱动器
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        public  Task<ExecutionResult> QueryDriveInfo()
        {
            return _appUpdataServer.GetDriveInfo();
        }
        
        /// <summary>
        /// 获取目录
        /// </summary>
        /// <param name="rootPath"></param>
        /// <returns></returns>
        [HttpGet]
        public async Task<ExecutionResult> QueryDirectoryInfo(string rootPath)
        {
           return await _appUpdataServer.GetDirectoryAndFileInfoAsync(rootPath);
        }

        /// <summary>
        /// 获取目录
        /// </summary>
        /// <param name="rootPath"></param>
        /// <returns></returns>
        [HttpGet]
        public async Task<ExecutionResult> QueryAllDirectoryAndFileInfo(string rootPath)
        {
            return await _appUpdataServer.GetAllDirectoryAndFileInfoAsync(rootPath);
        }

        [HttpGet]
        public Task<ExecutionResult> Download(string url, string fileName, string userName)
        {
            return _UploadfileServer.DownloadAndSaveFileInfo(url, fileName, userName);
        }
       
        /// <summary>
        /// 下载
        /// </summary>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        [HttpGet]
        public  FileResult DownloadByPath([Required]string path)
        {
            
            if (System.IO.File.Exists(path))
            {
                FileInfo f = new FileInfo(path);
                var fileStream = new FileStream(path, FileMode.Open, FileAccess.ReadWrite);
                return File(fileStream, "application/octet-stream", f.Name);
            }
            else
            {
                return null;
            }
          
        }

        /// <summary>
        /// 上传文件
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public async Task<ExecutionResult> UploadFileByPath()
        {
            var files = Request.Form.Files;
            if (files.Count <= 0)
                return ExecutionResult.Failed("上传文件不能为空");
         //   var replace =bool.Parse(Request.Form["replace"].ToString());
            var path =Request.Form["path"].ToString();
            /*if (string.IsNullOrEmpty(path))
                return ExecutionResult.Failed("保存路径不能为空");
            if (!Directory.Exists(path) && !replace)
                return ExecutionResult.Failed("保存路径不存在");*/
            foreach (var file in files)
            {
              
                if (!Directory.Exists(path) )
                {
                    Directory.CreateDirectory(path);
                }
                var filePath = Path.Combine(path, file.FileName.ToString());
                // tmpfile.HashID = s.GetHashCode().ToString();
                using (var fileStream = new FileStream(filePath, FileMode.OpenOrCreate, FileAccess.ReadWrite))
                {
                    await file.CopyToAsync(fileStream);
                    fileStream.Flush(true);
                }
            }
            return ExecutionResult.Success("文件保存成功");
        }

        /// <summary>
        /// 小文件上传接口
        /// </summary>
        ///  <remarks>
        /// 需要请求头user ：登录用户 必输
        ///     请求头Content-Type：multipart/form-data 必输
        ///     请求头app：上传文件的程序 必输
        ///     请求头UploadFolder ：指定存放文件加可以为空。为空时默认为Default
        /// </remarks>
        /// <returns></returns>
        [HttpPost]
        public async Task<ExecutionResult> Upload()
        {
            //_logger.LogInformation("jies");
            if (Request.Form.Files.Count <= 0)
                return ExecutionResult.Failed("请添加文件");

            if (!Request.Headers.ContainsKey("user"))
                return ExecutionResult.Failed("没有权限");

            var contentType = Request.Headers["Content-Type"].ToString();

            if (!contentType.Contains("multipart/form-data"))
                return ExecutionResult.Failed("ContentType内容必须是multipart/form-data");
            if (!Request.Headers.ContainsKey("app"))
                return ExecutionResult.Failed("未获取到请求头[APP]");

            var app = Request.Headers["app"].ToString();
            if (string.IsNullOrEmpty(app))
                return ExecutionResult.Failed("未获取到请求头[APP]:程序名称 对应的值");
            var UploadFolder = Request.Headers["UploadFolder"].ToString();

            return await _UploadfileServer.UploadAndSaveFileInfo(new UplaodFileInfoDto
            {
                app = app,
                contentType = contentType,
                files = Request.Form.Files,
                UploadFolder = UploadFolder ?? "Default",
                UserName = Request.Headers["user"]
            });
        }
    }
}