﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using OrBit.MESInterface.Application.Contracts.Hikvision;
using OrBit.MESInterface.Application.Contracts.Interfaces;
using OrBit.MESInterface.Application.Contracts.WGTask;
using OrBit.MESInterface.Application.Service.MESInterface;
using OrBit.MESInterface.HttpApi.Host.Base.Attribute;
using OrBit.MESInterface.Infrastructure;
using System;
using System.IO;
using System.Threading.Tasks;
using Volo.Abp.AspNetCore.Mvc;

namespace OrBit.MESInterface.HttpApi.Host.Controllers
{
    /// <summary>
    /// OrBitMes接口网关
    /// </summary>
    [ApiController]
    [Route("api/Hikvision/[action]")]
    public class OrBitHikvisionControllers : AbpController
    {
        private readonly IHikvisionGatewayAppService hikvisionGatewayAppService;
        private readonly ILogger<OrBitHikvisionControllers> _logger;

        /// <summary>
        /// OrBitMes接口网关
        /// </summary>
        /// <param name="downloadDataAppService"></param>
        public OrBitHikvisionControllers(IHikvisionGatewayAppService hikvisionGatewayAppService, ILogger<OrBitHikvisionControllers> logger)
        {
            this.hikvisionGatewayAppService = hikvisionGatewayAppService;
            _logger = logger;
        }


/// <summary>
/// 海康威视接口网关
/// </summary>
/// <param name="InterfaceType"></param>
/// <param name="InterfaceServiceName"></param>
/// <param name="appkey"></param>
/// <param name="secret"></param>
/// <param name="RequestDataParameters"></param>
/// <param name="UploadID"></param>
/// <returns></returns>
        [HttpGet]
        [NoSign]
        public async Task<string> Gateway(string InterfaceType, string InterfaceServiceName, string appkey, string secret, string RequestDataParameters = "",  string UploadID = "")
        {
            var EventID = UploadID;           

            _logger.LogInformation($"EventID:{EventID}MES接口[Gateway]接受到参数:InterfaceType:[{InterfaceType}],InterfaceServiceName:{InterfaceServiceName},EventID:{UploadID},RequestDataParameters:{RequestDataParameters} ");
            var result = await hikvisionGatewayAppService.Gateway(InterfaceType, InterfaceServiceName, appkey,secret, RequestDataParameters, EventID);
            // if (result.State == Application.Contracts.Dtos.AnalyticState.SaveTooBusinessTableSuccess)
            return JsonConvert.SerializeObject(result);
        }
    }
}