﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using OrBit.MESInterface.Application.Contracts;
using OrBit.MESInterface.Application.Contracts.WebDialog;
using OrBit.MESInterface.Application.Contracts.WebDialog.Dtos;
using OrBit.MESInterface.HttpApi.Host.Base.Attribute;
using System.Threading.Tasks;
using OrBit.MESInterface.Infrastructure;
using Volo.Abp.AspNetCore.Mvc;

namespace OrBit.MESInterface.HttpApi.Host.Controllers
{
    /// <summary>
    /// OrBitMes接口网关
    /// </summary>
    [ApiController]
    [Route("api/OrBitMESWebDialog/[action]")]
    public class OrBitMESWebDialogControllers : AbpController
    {
        private readonly ILogger<OrBitMESWebDialogControllers> _logger;

        private readonly IQueryWebDialogInfoAppService _QueryWebDialogInfo;

        private readonly IWebDialogPutDataAppService _webDialogPutDataAppService;

        public OrBitMESWebDialogControllers(ILogger<OrBitMESWebDialogControllers> logger, IQueryWebDialogInfoAppService QueryWebDialogInfo, IWebDialogPutDataAppService webDialogPutDataAppService)
        {
            _logger = logger;
            _QueryWebDialogInfo = QueryWebDialogInfo;
            _webDialogPutDataAppService = webDialogPutDataAppService;
        }

        /// <summary>
        /// 初始化对话框
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
      //  [BasicAuth]
        [HttpPost]
        [NoSign]
        public async Task<ExecutionResult> Initialization(OrBitUserDialogPlugInInfoDto dto)
        {
            var ip = Request.HttpContext.Connection.RemoteIpAddress.MapToIPv4().ToString();

            _logger.LogInformation($"接受IP:[{ip}]对MES接口[PutData]的请求。参数:{dto.ButtonName}{dto.PluginCommand}");
            return await _QueryWebDialogInfo.QueryWebDialogInfo(dto);
        }

        [HttpPost]
        [NoSign]
        public async Task<ExecutionResult> PutData(PutDataDto dto)
        {
            var ip = Request.HttpContext.Connection.RemoteIpAddress.MapToIPv4().ToString();
            _logger.LogInformation($"接受IP:[{ip}]对MES接口[PutData]的请求。参数:按钮[{dto.ButtonName}]命令[{dto.PluginCommand}]存储过程[{dto.StoredProcedureName}]参数{JsonConvert.SerializeObject(dto.parameters)}");

            return await _webDialogPutDataAppService.ExecutionSstoredProcedure(dto);
        }
    }
}