﻿using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using NUglify;
using OrBit.MESInterface.Application.Contracts.Interfaces;
using OrBit.MESInterface.Application.Contracts.WGTask;
using OrBit.MESInterface.Application.Service.MESInterface;
using OrBit.MESInterface.HttpApi.Host.Base.Attribute;
using OrBit.MESInterface.Infrastructure;
using OrBit.MESInterface.Infrastructure.Common;
using System;
using System.IO;
using System.Threading.Tasks;
using Volo.Abp.AspNetCore.Mvc;

namespace OrBit.MESInterface.HttpApi.Host.Controllers
{
    /// <summary>
    /// OrBitMes接口网关
    /// </summary>
    [ApiController]
    [Route("api/OrBitMesInterface/[action]")]
    public class OrBitMesInterfaceControllers : AbpController
    {
        private readonly IDownloadDataAppService _downloadDataAppService;
        private readonly ILogger<OrBitMesInterfaceControllers> _logger;

        private readonly PutDataAppService _PutDataAppService;
        private readonly ISendWGTaskService sendWGTaskService;

       
        /// <summary>
        /// OrBitMes接口网关
        /// </summary>
        /// <param name="downloadDataAppService"></param>
        public OrBitMesInterfaceControllers(PutDataAppService _PutDataAppService,IDownloadDataAppService downloadDataAppService, ILogger<OrBitMesInterfaceControllers> logger, ISendWGTaskService sendWGTaskService)
        {
            _downloadDataAppService = downloadDataAppService;
            _logger = logger;
            this.sendWGTaskService= sendWGTaskService;
            this._PutDataAppService = _PutDataAppService;
        }

      
        [HttpGet]        
        public async Task<ExecutionResult>SendWGTask(string assetNumber, string productName,string materialGroupID,string timeBlowOut, bool distributePrograms = true)
        {
            var ip = Request.HttpContext.Connection.RemoteIpAddress.MapToIPv4().ToString();
            var id = Guid.NewGuid().ToString();
            _logger.LogInformation($"EventID:{id}MES接口[Gateway]接受到参数:materialGroupID:{materialGroupID}, assetNumber:{assetNumber},timeBlowOut:{timeBlowOut}");
           
           return  await sendWGTaskService.Send(assetNumber, productName, materialGroupID, timeBlowOut, distributePrograms);
        }

        /// <summary>
        /// 通过GZIP压缩字符串
        /// </summary>
        /// <param name="dataSource">未压缩的字符串</param>
        /// <returns>压缩后的字符串</returns>
        [HttpPost]
        public async Task<string> GzipEncryption()
        {
            var RequestBody = string.Empty;

            using (var reader = new StreamReader(Request.Body))
            {
                RequestBody = await reader.ReadToEndAsync();
            }
            return GZipUtillib.GZipCompressString(RequestBody);
        }

        /// <summary>
        /// 将压缩的字符串解压缩后返回
        /// </summary>
        /// <param name="zippedString">压缩的字符串</param>
        /// <returns>解压缩的字符串</returns>
        [HttpPost]
        public async Task<string> GzipDecrypt()
        {
            var RequestBody = string.Empty;

            using (var reader = new StreamReader(Request.Body))
            {
                RequestBody = await reader.ReadToEndAsync();
            }

           // return Task.FromResult( GZipUtillib.GZipDecompressString(str));
            return GZipUtillib.GZipDecompressString(RequestBody);
        }

        /// <summary>
        /// OrBitMes接口网关
        /// </summary>
        /// <param name="InterfaceType">接口类型</param>
        /// <param name="InterfaceServiceName">接口服务名称</param>
        /// <param name="UploadID">上传ID</param>
        /// <param name="RequestDataParameters">请求参数</param>
        /// <returns></returns>
        [HttpGet]
        [NoSign]
        public async Task<string> Gateway(string InterfaceType, string InterfaceServiceName, string RequestDataParameters = "", string RequestURL = "", string UploadID = "")
        {
            var EventID = UploadID;
            if (InterfaceType.Equals("Download"))
            {
                EventID = Guid.NewGuid().ToString();
            }

            _logger.LogInformation($"EventID:{EventID}MES接口[Gateway]接受到参数:InterfaceType:[{InterfaceType}],InterfaceServiceName:{InterfaceServiceName},EventID:{UploadID},RequestDataParameters:{RequestDataParameters} ");
            var result = await _downloadDataAppService.SingleInterfaceDownload(InterfaceType, InterfaceServiceName, RequestURL, RequestDataParameters, EventID);
            // if (result.State == Application.Contracts.Dtos.AnalyticState.SaveTooBusinessTableSuccess)
            return JsonConvert.SerializeObject(result);
        }

        /// <summary>
        /// 推送接口
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>

        // [BasicAuth]
        [HttpPost]
        public async Task<ExecutionResult> PutData()
        {
            var ip = Request.HttpContext.Connection.RemoteIpAddress.MapToIPv4().ToString();
            var id = Guid.NewGuid().ToString();

            var ConsumerId = Request.Headers["ConsumerId"];
            if (string.IsNullOrEmpty(ConsumerId))
            {
                return ExecutionResult.Failed("当前请求没有给定ConsumerId！");
            }

            if (Request.Headers.ContentType.ToString().Equals("application/json"))
            {
                var RequestBody = string.Empty;

                using (var reader = new StreamReader(Request.Body))
                {
                    RequestBody = await reader.ReadToEndAsync();
                }

                _logger.LogInformation($"请求事件id:{id},接受IP:[{ip}]对MES接口[PutData]的请求。参数:{RequestBody}");
                return await _PutDataAppService.PutJsonDataSaveAsync(RequestBody, ConsumerId, HttpContext, id);
            }
            else
            {
                _logger.LogInformation($"请求事件id:{id},接受IP:[{ip}]对MES接口[PutData]的文件上传请求。");
                return await _PutDataAppService.PutFileDataSaveAsync(HttpContext, ConsumerId, id);
            }
        }

        /// <summary>
        /// 业务系统推送传输内容到数据中心通知接收接口
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public async Task<CustomizationResult> ReciveMsg()
        {
            var ip = Request.HttpContext.Connection.RemoteIpAddress.MapToIPv4().ToString();
            var id = Guid.NewGuid().ToString();
            
            //if (Request.Headers.ContentType.ToString().Equals("application/json"))
            //{
            return await GetBodyData("DataCenterReciveMsg", Request, (data, id) =>
            {
                return _PutDataAppService.PutJsonDataSaveAsync("DataCenterReciveMsg", data, "", Request.HttpContext, id).Result;
            });
            //}
            //else
            //{
            //    _logger.LogInformation($"接受IP:[{ip}]对MES接口[ReciveMsg]的请求。ContentType为不支持的内容，请确保ContentType是application/json");
            //    return new CustomizationResult { success = false, message = "ContentType为不支持的内容，请确保ContentType是application/json" };

            //}
        }

        /// <summary>
        /// 自调用拼装json
        /// </summary>
        /// <param name="InterafaceName">服务内容</param>
        /// <param name="Request">请求数据</param>
        /// <param name="PutData">自调用方法</param>
        /// <returns></returns>
        private async Task<CustomizationResult> GetBodyData(string InterafaceName, HttpRequest Request, Func<string, string, ExecutionResult> PutData)
        {
            try
            {
                var RequestBody = string.Empty;
                using (var reader = new StreamReader(Request.Body))
                {
                    RequestBody = await reader.ReadToEndAsync();
                }
                var ip = Request.HttpContext.Connection.RemoteIpAddress.MapToIPv4().ToString();
                var id = Guid.NewGuid().ToString();
                _logger.LogInformation($"请求事件id:{id},接受IP:[{ip}]对MES接口[{InterafaceName}]的请求。参数:{RequestBody}");


                if (string.IsNullOrWhiteSpace(RequestBody))
                {
                    return new CustomizationResult { success = false, message = "请求参数错误" };
                }
                //json 与 事件id
                var result = PutData(RequestBody, id);
                return new CustomizationResult
                {
                    success= result.code== ExecutionState.SUCCESS?true:false,
                    message= result.message
                };
            }
            catch
            {
                return new CustomizationResult { success = false, message = "请求参数错误" };
            }

        }
    }

  
    public class CustomizationResult
    {
       
        public bool success { get; set; }
        public string message { get; set; }

    }
}