﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using OrBit.MESInterface.Application.Contracts.RemontRepositorie;
using OrBit.MESInterface.Application.Contracts.RemontRepositorie.Dto;
using OrBit.MESInterface.Infrastructure;
using System;
using System.IO;
using System.Threading.Tasks;
using Volo.Abp.AspNetCore.Mvc;

namespace OrBit.MESInterface.HttpApi.Host.Controllers
{
    /// <summary>
    /// OrBitMes远程仓储服务
    /// </summary>
    [ApiController]
    [Route("api/RemontRepositorieService/[action]")]
    public class RemontRepositorieServiceControllers : AbpController
    {
        private readonly ILogger<RemontRepositorieServiceControllers> _logger;
        private readonly IRemontRepositorService remontRepositorService;

        public RemontRepositorieServiceControllers(ILogger<RemontRepositorieServiceControllers> _logger
           ,IRemontRepositorService remontRepositorService)
        {
             this._logger = _logger;
            this.remontRepositorService = remontRepositorService;
        }

        /// <summary>
        /// 执行sql
        /// </summary>
        ///  <remarks>
        /// 需要请求头user ：登录用户 必输
        ///     请求头Content-Type：application/json 必输
        ///     请求头app：执行程序 必输
        /// </remarks>
        /// <returns></returns>
        [HttpPost]
        public async Task<ExecutionResult> RunSql()
        {
            //_logger.LogInformation("jies");
            if (!Request.Headers.ContainsKey("user"))
                return ExecutionResult.Failed("没有权限");
            var contentType = Request.Headers["Content-Type"].ToString();
            if (!contentType.Contains("application/json"))
                return ExecutionResult.Failed("ContentType内容必须是application/json");
            if (!Request.Headers.ContainsKey("app"))
                return ExecutionResult.Failed("未获取到请求头[APP]");

            var app = Request.Headers["app"].ToString();
            if (string.IsNullOrEmpty(app))
                return ExecutionResult.Failed("未获取到请求头[APP]:程序名称 对应的值");

          
            if (Request.Headers.ContentType.ToString().Equals("application/json"))
            {
                var RequestBody = string.Empty;

                using (var reader = new StreamReader(Request.Body))
                {
                    RequestBody = await reader.ReadToEndAsync();
                }
                var userName = Request.Headers["user"].ToString();
                var id = Guid.NewGuid().ToString();
                var ip = Request.HttpContext.Connection.RemoteIpAddress.MapToIPv4().ToString();
                _logger.LogInformation($"请求事件id:{id},接受IP:[{ip}]对MES接口[PutData]的请求。参数:{RequestBody}");

              return  await remontRepositorService.QueryData(JsonConvert.DeserializeObject<RemontRepositorieCommandInfoDto>(RequestBody), userName, ip);
                //   return await _PutDataAppService.PutJsonDataSaveAsync(RequestBody, ConsumerId, HttpContext, id);
            }

            return ExecutionResult.Failed("参数不正确");
         
        }
    }
}