﻿using Autofac.Core;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Http.Features;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Server.Kestrel.Core;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.OpenApi.Models;
using OrBit.MESInterface.Application;
using OrBit.MESInterface.Application.Contracts;
using OrBit.MESInterface.EntityFrameworkCore;
using OrBit.MESInterface.HttpApi.Host.Base.Extensions;
using OrBit.MESInterface.HttpApi.Host.Base.Filters;
using OrBit.MESInterface.Infrastructure.Common;
using System;
using System.Configuration;
using System.IO;
using System.Reflection;
using OrBit.MESInterface.Application.Contracts.ApiFile.Dtos;
using Volo.Abp;
using Volo.Abp.AspNetCore.ExceptionHandling;
using Volo.Abp.AspNetCore.Mvc;
using Volo.Abp.Autofac;
using Volo.Abp.Modularity;
using OrBit.MESInterface.Infrastructure.Explorer;

namespace OrBit.MESInterface.HttpApi.Host
{
    [DependsOn(
        typeof(AbpAutofacModule),
        typeof(AbpAspNetCoreMvcModule),
        typeof(MESInterfaceApplicationModule),
         typeof(MESInterfaceApplicationContractsModule),
        typeof(MESInterfaceEntityFrameworkModule)
        )]
    public class MESIntefaceHttpApiHostModule : AbpModule
    {
        public override void ConfigureServices(ServiceConfigurationContext context)
        {
            context.Services.AddSingleton<SnowflakeTool>();

            context.Services.Configure<KestrelServerOptions>(options =>
            {
                // Set the limit to 512 MB
                options.Limits.MaxRequestBodySize = 536346624;
            });
            context.Services.Configure<IISServerOptions>(options =>
            {
                options.MaxRequestBodySize = 536346624;
            });
            context.Services.Configure<FormOptions>(x => x.MultipartBodyLengthLimit = 536346624);
            // 添加跨域
            context.Services.AddCors(options =>
            {
                options.AddPolicy("CorsPolicy", builder =>
                {
                    builder.AllowAnyOrigin()
                           .AllowAnyMethod()
                           .AllowAnyHeader();
                });
            });
            context.Services.Configure<UploadFileConfig>(context.Services.GetConfiguration().GetSection("UploadFileConfig"));
            context.Services.AddHttpClient();
            context.Services.AddHttpClient<DefaultHttpClientHelper>(option =>
            {
                option.Timeout = TimeSpan.FromMinutes(10);
            });

            context.Services.AddSwaggerGen(options =>
            {
                options.SwaggerDoc("v1", new OpenApiInfo { Title = "OrBit Interface API", Version = "v1" });
                options.DocInclusionPredicate((docName, description) => true);
                options.CustomSchemaIds(type => type.FullName);
                var xmlFile = $"{Assembly.GetExecutingAssembly().GetName().Name}.XML";
                var xmlPath = Path.Combine(AppContext.BaseDirectory, xmlFile);
                options.IncludeXmlComments(xmlPath);
            });
            context.Services.AddTransient<ExplorerMiddleware>();
            context.Services.AddTransient<IExplorer,DefaultExplorerProvider>();
            Configure<AbpExceptionHandlingOptions>(options =>
            {
                options.SendExceptionsDetailsToClients = false;
            });
            Configure<MvcOptions>(options =>
            {
                options.Filters.Add(typeof(GlobalExceptionFilter));
            });
        }

        public override void OnApplicationInitialization(ApplicationInitializationContext context)
        {
            var app = context.GetApplicationBuilder();
            var env = context.GetEnvironment();

            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
                app.UseSwagger();
                app.UseSwaggerUI(options =>
                {
                    options.SwaggerEndpoint("/swagger/v1/swagger.json", "OrBit Interface API");
                });
            }
            app.UseWhen(
          context => context.Request.Path.StartsWithSegments("/Explorer",
              StringComparison.OrdinalIgnoreCase),
          appBuilder => { appBuilder.UseMiddleware<ExplorerMiddleware>(); });
            app.UseRouting();

            app.UseCors("CorsPolicy");
            app.UseOrBitAuthorization();

            app.UseUploadDataIDFilter();
            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllerRoute(
                    name: "default",
                    pattern: "api/{controller=Home}/{action=Index}/{id?}");
            });
        }
    }
}