using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Hosting;
using Serilog;
using Serilog.Events;
using Serilog.Sinks.MSSqlServer;
using System.IO;

namespace OrBit.MESInterface.HttpApi.Host
{
    public class Program
    {
        public static void Main(string[] args)
        {
            var configuration = new ConfigurationBuilder()
                   .SetBasePath(Directory.GetCurrentDirectory())
                   .AddJsonFile("appsettings.json")
                   .Build();
            //Log.Logger = new LoggerConfiguration()
            //     .ReadFrom.Configuration(configuration)
            //     .CreateLogger();

            var columnOpts = new ColumnOptions();
            columnOpts.Store.Remove(StandardColumn.Properties);
            columnOpts.Store.Add(StandardColumn.LogEvent);
            columnOpts.LogEvent.DataLength = 2048;
            //  columnOpts.PrimaryKey = options.TimeStamp;
            columnOpts.TimeStamp.NonClusteredIndex = true;

            var mssqloption = new MSSqlServerSinkOptions();
            mssqloption.TableName = configuration.GetSection("Serilog:MSSqlServer:TableName").Value;
            mssqloption.BatchPostingLimit = configuration.GetValue<int>("Serilog:MSSqlServer:BatchPostingLimit");
            mssqloption.AutoCreateSqlTable = true;

            string Logformat = @"{Timestamp:yyyy-MM-dd HH:mm:ss }[{Level:u3}] {Message:lj}{NewLine}";
            //ƴһܵ
            Log.Logger = new LoggerConfiguration()
                //͵ȼ
                .MinimumLevel.Information()
                  .MinimumLevel.Override("Microsoft", Serilog.Events.LogEventLevel.Error)
                .MinimumLevel.Override("Microsoft.Hosting.Lifetime", LogEventLevel.Error)
                .MinimumLevel.Override("Volo", Serilog.Events.LogEventLevel.Error)
                .Enrich.FromLogContext()
                //¼͵̨չʾ
                .WriteTo.Async(c => c.Console(outputTemplate: Logformat))
                .WriteTo.Async(c => c.File("Logs/Log.txt", rollingInterval: RollingInterval.Day, outputTemplate: Logformat))
                //.WriteTo.Async(c => c.MSSqlServer(connectionString: configuration.GetSection("Serilog:MSSqlServer:ConnectionString").Value,
                //                    sinkOptions: mssqloption,
                //                    restrictedToMinimumLevel: LogEventLevel.Information,
                //                    columnOptions: columnOpts
                //                    ))
                .CreateLogger();

            CreateHostBuilder(args).Build().Run();
        }

        public static IHostBuilder CreateHostBuilder(string[] args) =>
            Microsoft.Extensions.Hosting.Host.CreateDefaultBuilder(args)
                .ConfigureWebHostDefaults(webBuilder =>
                {
                    webBuilder.UseStartup<Startup>();
                })
            .UseAutofac()
            .UseSerilog();
    }
}