﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http.Headers;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;
using System.Security.Cryptography;

namespace OrBit.MESInterface.Infrastructure.Common
{
    /// <summary>
/// 调用海康威视接口参数
/// </summary>
    public class HikvisonHttpUtillibDto
    {
        public HikvisonHttpUtillibDto()
        {
            Prot = "443";
            IsHtttps= true;
        }
        /// <summary>
        /// 请求路径
        /// </summary>
        public string Path { get; set; }

        /// <summary>
        /// 请求的数据json
        /// </summary>
        public string Body { get; set; }

        public string Appkey { get; set; }

        public string Secret { get; set; }
        /// <summary>
        /// 端口号
        /// </summary>
        public string Prot { get; set; }
        /// <summary>
        /// 是否使用htttps
        /// </summary>
        public bool IsHtttps { get; set; } 
    }

    /// <summary>
    /// 调用海康威视接口帮助类
    /// </summary>
    public class HikvisonHttpUtillib
    {
        public async Task<ExecutionResult> Post( string url, HikvisonHttpUtillibDto hikvisonHttpUtillibDto)
        {

            //StringBuilder sb = new StringBuilder();
            //sb.Append(isHttps ? "https://" : "http://").Append(ip).Append(":").Append(prot).Append(uri);
            // 初始化请求：组装请求头，设置远程证书自动验证通过
            var header = BuildHeader(hikvisonHttpUtillibDto.Path, hikvisonHttpUtillibDto.Body, true, hikvisonHttpUtillibDto.Appkey, hikvisonHttpUtillibDto.Secret);
            var clientHandler = new HttpClientHandler
            {
                AllowAutoRedirect = false,
                ServerCertificateCustomValidationCallback = (sender, cert, chain, sslPolicyErrors) => { return true; },

            };          
            using (HttpClient client = new HttpClient(clientHandler))
            {
                client.Timeout = TimeSpan.FromSeconds(15 * 1000);
                client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
                var stringContent = new StringContent(hikvisonHttpUtillibDto.Body, Encoding.UTF8);
                foreach (string headerKey in header.Keys)
                {
                    if (headerKey.Contains("x-ca-"))
                    {
                        stringContent.Headers.Add(headerKey, header[headerKey]);
                        //  await Console.Out.WriteLineAsync($"{headerKey}:{header[headerKey]}");
                    }
                }
                stringContent.Headers.ContentType = new MediaTypeWithQualityHeaderValue("application/json");
                var response = await client.PostAsync(url, stringContent);
              
                return ExecutionResult.Success($"{response.StatusCode}", await response.Content.ReadAsStringAsync());              
            }

        }
        private Dictionary<string, string> BuildHeader(string url, string body, bool isPost, string _appkey, string _secret)
        {
            Dictionary<string, string> header = new Dictionary<string, string>();
            // Accept                
            string accept = "application/json";// "*/*";
            header.Add("Accept", accept);

            // ContentType  
            string contentType = "application/json";
            header.Add("Content-Type", contentType);

            if (isPost)
            {
                // content-md5，be careful it must be lower case.
                string contentMd5 = computeContentMd5(body);
                header.Add("content-md5", contentMd5);
            }

            // x-ca-timestamp
            string ID = TimeZoneInfo.Local.Id;
            DateTime start = new DateTime(1970, 1, 1) + TimeZoneInfo.Local.GetUtcOffset(DateTime.Now);
            string timestamp = ((DateTime.Now.Ticks - TimeZoneInfo.ConvertTime(start, TimeZoneInfo.FindSystemTimeZoneById(ID)).Ticks) / 1000).ToString(); //((DateTime.Now.Ticks - TimeZone.CurrentTimeZone.ToLocalTime(new System.DateTime(1970, 1, 1, 0, 0, 0, 0)).Ticks) / 1000).ToString();
            header.Add("x-ca-timestamp", timestamp);

            // x-ca-nonce
            string nonce = System.Guid.NewGuid().ToString();
            header.Add("x-ca-nonce", nonce);

            // x-ca-key
            header.Add("x-ca-key", _appkey);

            // build string to sign
            string strToSign = buildSignString(isPost ? "POST" : "GET", url, header);
            string signedStr = computeForHMACSHA256(strToSign, _secret);

            // x-ca-signature
            header.Add("x-ca-signature", signedStr);

            return header;
        }
        /// <summary>
        /// 计算签名头
        /// </summary>
        /// <param name="header">请求头</param>
        /// <returns>签名头</returns>
        private string buildSignHeader(Dictionary<string, string> header)
        {
            Dictionary<string, string> sortedDicHeader = new Dictionary<string, string>();
            sortedDicHeader = header;
            var dic = from objDic in sortedDicHeader orderby objDic.Key ascending select objDic;

            StringBuilder sbSignHeader = new StringBuilder();
            StringBuilder sb = new StringBuilder();
            foreach (KeyValuePair<string, string> kvp in dic)
            {
                if (kvp.Key.Replace(" ", "").Contains("x-ca-"))
                {
                    sb.Append(kvp.Key + ":");
                    if (!string.IsNullOrWhiteSpace(kvp.Value))
                    {
                        sb.Append(kvp.Value);
                    }
                    sb.Append("\n");
                    if (sbSignHeader.Length > 0)
                    {
                        sbSignHeader.Append(",");
                    }
                    sbSignHeader.Append(kvp.Key);
                }
            }

            header.Add("x-ca-signature-headers", sbSignHeader.ToString());

            return sb.ToString();
        }

        /// <summary>
        /// 计算签名字符串
        /// </summary>
        /// <param name="method">HTTP请求方法，如“POST”</param>
        /// <param name="url">接口Url，如/artemis/api/resource/v1/org/advance/orgList</param>
        /// <param name="header">请求头</param>
        /// <returns>签名字符串</returns>
        private string buildSignString(string method, string url, Dictionary<string, string> header)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(method.ToUpper()).Append("\n");
            if (null != header)
            {
                if (null != header["Accept"])
                {
                    sb.Append((string)header["Accept"]);
                    sb.Append("\n");
                }

                if (header.Keys.Contains("Content-MD5") && null != header["Content-MD5"])
                {
                    sb.Append((string)header["Content-MD5"]);
                    sb.Append("\n");
                }

                if (null != header["Content-Type"])
                {
                    sb.Append((string)header["Content-Type"]);
                    sb.Append("\n");
                }

                if (header.Keys.Contains("Date") && null != header["Date"])
                {
                    sb.Append((string)header["Date"]);
                    sb.Append("\n");
                }
            }

            // build and add header to sign
            string signHeader = buildSignHeader(header);
            sb.Append(signHeader);
            sb.Append(url);
            return sb.ToString();
        }

        /// <summary>
        /// 计算HMACSHA265
        /// </summary>
        /// <param name="str">待计算字符串</param>
        /// <param name="secret">平台APPSecet</param>
        /// <returns>HMAXH265计算结果字符串</returns>
        private string computeForHMACSHA256(string str, string secret)
        {
            var encoder = new System.Text.UTF8Encoding();
            byte[] secretBytes = encoder.GetBytes(secret);
            byte[] strBytes = encoder.GetBytes(str);
            var opertor = new HMACSHA256(secretBytes);
            byte[] hashbytes = opertor.ComputeHash(strBytes);
            return Convert.ToBase64String(hashbytes);
        }


        /// <summary>
        /// 计算content-md5
        /// </summary>
        /// <param name="body"></param>
        /// <returns>base64后的content-md5</returns>
        private string computeContentMd5(string body)
        {
            MD5 md5 = MD5.Create();           
            byte[] result = md5.ComputeHash(System.Text.Encoding.UTF8.GetBytes(body));
            return Convert.ToBase64String(result);
        }
    }
}
