﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Data;
using System.Reflection.PortableExecutable;
using System.Text.RegularExpressions;

namespace OrBit.MESInterface.Infrastructure.Common
{
    public class JsonHelper
    {
        /// <summary>
        /// 将JSON转数组
        /// 用法:jsonArr[0]["xxxx"]
        /// </summary>
        /// <param name="json">json字符串</param>
        /// <returns></returns>
        public static JArray GetToJsonList(string json, bool CastsJArray = true)
        {
            if (!isJArray(json) && !CastsJArray)
                return null;
            if (!isJArray(json) && CastsJArray && !isJObject(json))
                return null;

            if (!isJArray(json) && CastsJArray && isJObject(json)) //强转
            {
                json = $"[{JsonConvert.DeserializeObject(json)}]";
            }

            JArray jsonArr = (JArray)JsonConvert.DeserializeObject(json);
            return jsonArr;
        }

        public static JToken ReadJSON(string jsonStr)
        {
            try
            {
                JObject jobj = JObject.Parse(jsonStr);
                JToken result = jobj as JToken;
                return result;
            }
            catch
            {
                throw new Exception("字符串不是json");
            }
        }

        private static List<string> TemplateName = new List<string>();

        private static bool AnalysisJArray(JToken J, string AssignmentName, string NewValue, string ParentAssignmentName = "")
        {
            if (J.Type == JTokenType.Array)
            {
                for (int i = 0; i < ((JArray)J).Count; i++)
                {
                    if (AnalysisJObject(J[i], AssignmentName, NewValue, ParentAssignmentName))
                        return true;
                }
            }
            return false;
        }

        private static bool AnalysisJObject(JToken J, string AssignmentName, string NewValue, string ParentAssignmentName = "")
        {
            //Console.WriteLine("jtoken:"+J.ToString());
            //Console.WriteLine("--AssignmentName:" + AssignmentName);
            if (J.Type == JTokenType.Object)
            {
                foreach (JProperty property in ((JObject)J).Properties())
                {
                    if (TemplateName.Contains(AssignmentName))
                        continue;

                    var tmpAssignmentName = AssignmentName;
                    if (tmpAssignmentName.Contains("$"))
                    {
                        ParentAssignmentName = tmpAssignmentName.Substring(0, tmpAssignmentName.IndexOf("$"));
                        AssignmentName = tmpAssignmentName.Substring(tmpAssignmentName.IndexOf("$") + 1, tmpAssignmentName.Length - ParentAssignmentName.Length - 1);
                    }

                    // Console.WriteLine($"property.Type：{property.Type}，Name：{property.Name},value:{property.Value},property.Value.Type：{property.Value.Type}{"\n"}");
                    if (property.Name.Equals(AssignmentName) && (string.IsNullOrEmpty(ParentAssignmentName) || (property.Path.Contains(ParentAssignmentName) && !string.IsNullOrEmpty(ParentAssignmentName))))
                    {
                        if (isJObject(NewValue))
                            property.Value = JObject.Parse(NewValue);
                        else if (isJArray(NewValue))
                            property.Value = JArray.Parse(NewValue);
                        else
                            property.Value = NewValue;
                        //  Console.WriteLine($"property.Type：{property.Type}，Name：{property.Name},newValue:{property.Value},property.Value.Type：{property.Value.Type}{"\n"}");
                        return true;
                    }
                    if (property.Value.Type == JTokenType.Object)
                    {
                        var result = AnalysisJObject(property.Value, AssignmentName, NewValue, ParentAssignmentName);
                        if (result)
                            return result;
                    }
                    if (property.Value.Type == JTokenType.Array)
                    {
                        var result = AnalysisJArray(property.Value, AssignmentName, NewValue, ParentAssignmentName);
                        if (result)
                            return result;
                    }
                }
            }
            return false;
        }

        private static bool isJObject(string json)
        {
            if (string.IsNullOrEmpty(json))
            {
                return false;
            }

            try
            {
                JObject.Parse(json);
                return true;
            }
            catch
            {
                return false;
            }
        }

        public static bool isJArray(string json)
        {
            if (string.IsNullOrEmpty(json))
            {
                return false;
            }

            try
            {
                JArray.Parse(json);
                return true;
            }
            catch
            {
                return false;
            }
        }

        public static string JsonAssignment(string json, string key, string value)
        {
            JToken result = ReadJSON(json) as JToken;//转换为JToken
            JToken result2 = result.DeepClone();//复制一个返回值，由于遍历的时候JToken的修改回终止遍历，因此需要复制一个新的返回json
            //替换json
            Regex reg = new Regex(@"" + key + "$");
            //SelectToken(Path)方法可查找某路径下的节点，在Newtonsoft.Json 4.5 版本中不可使用正则匹配，在6.0版本中可用使用，会方便很多，6.0版本下替换值会更方便，这个需要特别注意的
            if (reg.IsMatch(key))
            {
                result2.SelectToken(key).Replace(value);
            }

            return result2.ToString();
        }

        public static JObject JsonTemplateReplacementResultJobject(string json, DataTable dt)
        {
            JObject jObj = JObject.Parse(json);
            for (int i = 0; i < dt.Columns.Count; i++)
            {
                //  var AssignmentName = dt.Columns[i].ColumnName;

                if (AnalysisJObject(jObj, dt.Columns[i].ColumnName, dt.Rows[0][i].ToString()))
                {
                    TemplateName.Add(dt.Columns[i].ColumnName);
                }
            }
            TemplateName.Clear();
            return jObj;
        }

        public static string JsonTemplateReplacement_Pro(string json, DataTable dt)
        {
            JObject jObj = JObject.Parse(json);
            for (int i = 0; i < dt.Columns.Count; i++)
            {
                //  var AssignmentName = dt.Columns[i].ColumnName;

                if (AnalysisJObject(jObj, dt.Columns[i].ColumnName, dt.Rows[0][i].ToString()))
                {
                    TemplateName.Add(dt.Columns[i].ColumnName);
                }
            }
            TemplateName.Clear();
            return jObj.ToString();
        }

        /// <summary>
        /// 遍历所以节点，替换其中某个节点的值
        /// </summary>
        /// <param name="jobj">json数据</param>
        /// <param name="nodeName">节点名</param>
        /// <param name="value">新值</param>
        public static string JsonTemplateReplacement(string json, DataTable dt)
        {
            try
            {
                JToken result = ReadJSON(json) as JToken;//转换为JToken
                JToken result2 = result.DeepClone();//复制一个返回值，由于遍历的时候JToken的修改回终止遍历，因此需要复制一个新的返回json
                                                    //遍历
                var reader = result.CreateReader();

                while (reader.Read())
                {
                    if (reader.Value != null)
                    {
                        if (reader.TokenType == JsonToken.String || reader.TokenType == JsonToken.Integer || reader.TokenType == JsonToken.Float)
                        {
                            var path = reader.Path;//获取到节点路径
                            var key = "";

                            //拿到节点
                            if (path.Contains("."))
                            {
                                key = path.Substring(path.LastIndexOf('.') + 1, path.Length - (path.Substring(0, path.LastIndexOf('.')).Length) - 1);
                            }
                            else
                                key = path;//.Substring(path.LastIndexOf('.') + 1, path.Length - (path.Substring(0, path.LastIndexOf('.')).Length) - 1);

                            //拿到模板默认值
                            var oldValue = reader.Value.ToString().Replace('{', ' ').Replace('}', ' ').TrimEnd().TrimStart();

                            //拿到新值
                            var newValue = string.Empty;

                            //如果这个key存在就拿出新值
                            if (dt.Columns.Contains(key))
                            {
                                newValue = dt.Rows[0][key].ToString();
                            }

                            //替换默认值
                            if (oldValue.Contains("=") && string.IsNullOrEmpty(newValue))
                            {
                                newValue = oldValue.Substring(oldValue.IndexOf('=') + 1, oldValue.Length - (oldValue.Substring(0, oldValue.IndexOf('=')).Length) - 1);
                            }

                            //替换json
                            Regex reg = new Regex(@"" + key + "$");
                            //SelectToken(Path)方法可查找某路径下的节点，在Newtonsoft.Json 4.5 版本中不可使用正则匹配，在6.0版本中可用使用，会方便很多，6.0版本下替换值会更方便，这个需要特别注意的
                            if (reg.IsMatch(reader.Path))
                            {
                                result2.SelectToken(reader.Path).Replace(newValue);
                            }

                            //else
                            //    result2.SelectToken(reader.Path).Value(= dt.Rows[0][key].ToString()
                        }
                    }
                    else
                    {
                    }
                }
                return result2.ToString();
            }
            catch (Exception ex)
            {
                // _logger.LogError(ex, ex.Message);
                throw ex;
            }
        }

        /// <summary>
        /// 删除或修改 单个节点
        /// </summary>
        /// <param name="section">节点途径 格式为A:B:C  如果节点带数组格式  A:[0]:C 数组用中括号</param>
        /// <param name="isRemove">是删除还是修改</param>
        /// <param name="value">修改节点的内容</param>
        /// <param name="isNum">修改内容类型 数据:true  字符串：false</param>
        private void JTokenHander(string json, string section, bool isRemove, string value, bool isNum)
        {
            try
            {
                JToken jObj = JToken.Parse(json);
                string[] nodes = section.Split(':');
                JToken tempToken = jObj;
                if (nodes != null && nodes.Length > 0)
                {
                    foreach (string node in nodes)
                    {
                        if (node != null)
                        {
                            if (node.StartsWith("[") && node.EndsWith("]"))
                            {
                                int tempi = 0;
                                if (int.TryParse(node.Trim('[').Trim(']'), out tempi))
                                {
                                    tempToken = tempToken[tempi];
                                }
                            }
                            else
                            {
                                tempToken = tempToken[node];
                            }
                        }
                        else
                        {
                            break;
                        }
                    }
                }
                if (tempToken != null)
                {
                    if (isRemove)
                    {
                        tempToken.Parent.Remove();
                    }
                    else
                    {
                        JToken token;
                        if (isNum)
                        {
                            if (value.Contains("."))
                            {
                                double tempd;
                                double.TryParse(value, out tempd);
                                token = tempd;
                            }
                            else
                            {
                                int tempi = 0;
                                int.TryParse(value, out tempi);
                                token = tempi;
                            }
                        }
                        else
                        {
                            token = value;
                        }
                        tempToken.Replace(token);//改
                    }
                }
            }
            catch (System.Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// 将datatable转换为json
        /// </summary>
        /// <param name="dtb">Dt</param>
        /// <returns>JSON字符串</returns>
        public static string Dtb2Json(DataTable dtb)
        {
            string JSONresult = JsonConvert.SerializeObject(dtb, Formatting.Indented);
            return JSONresult;//Json(new { JSONresult });
        }

        public static string StringArr2Json(DataTable dtb)
        {
            var stringArr = ToStringList(dtb);
            string JSONresult = JsonConvert.SerializeObject(stringArr, Formatting.Indented);
            return JSONresult;//Json(new { JSONresult });
        }

        public static List<string> ToStringList(List<Dictionary<string, object>> dt)
        {
            // 定义集合
            List<string> ts = new List<string>();
            if (dt == null || dt.Count <= 0)
                return ts;

            foreach (var d in dt)
            {
                foreach (var item in d.Keys)
                {
                    ts.Add(d[item].ToString().Trim());
                }
            }
            return ts;
        }

        /// <summary>
        /// 将json字符串反序列化为字典类型
        /// </summary>
        /// <typeparam name="TKey">字典key</typeparam>
        /// <typeparam name="TValue">字典value</typeparam>
        /// <param name="jsonStr">json字符串</param>
        /// <returns>字典数据</returns>
        public static Dictionary<string, object> DeserializeStringToDictionary(string jsonStr)
        {
            if (string.IsNullOrEmpty(jsonStr))
                return new Dictionary<string, object>();

            Dictionary<string, object> jsonDict = JsonConvert.DeserializeObject<Dictionary<string, object>>(jsonStr);

            return jsonDict;
        }

        /// <summary>
        /// 根据json字符串类型将json字符串反序列化为字典或者集合类型
        /// </summary>
        /// <param name="jsonStr"></param>
        /// <returns></returns>
        public static object DeserializeStringToObject(string jsonStr)
        {
            if (isJArray(jsonStr))
            {
                return JsonConvert.DeserializeObject<List<Dictionary<string, object>>>(jsonStr);
            }
            return DeserializeStringToDictionary(jsonStr);
        }

        public static List<string> ToStringList(DataTable dt)
        {
            // 定义集合
            List<string> ts = new List<string>();
            if (dt == null || dt.Rows.Count <= 0)
                return ts;

            foreach (DataRow dr in dt.Rows)
            {
                // 获得此模型的公共属性
                ts.Add(dr[0].ToString().Trim());
            }
            return ts;
        }
    }
}