﻿using System.Collections.Generic;
using System.Text;

namespace OrBit.MESInterface.Infrastructure.Common
{
        public enum RequestMethodType
        {
            GET,
            POST
        }

        public class RequestData
        {
            public RequestData()
            {
            }

            public RequestData(string Url, string RequestMethod, bool IsBasicAuth, string BasicAuthUser,
                               string BasicAuthPassword, IDictionary<string, string> Parameters, string PostData, IDictionary<string, string> Headers)
            {
                this.Url = Url;
                switch (RequestMethod.ToLower())
                {
                    case "post":
                        this.RequestMethod = RequestMethodType.POST;
                        break;

                    case "get":
                        this.RequestMethod = RequestMethodType.GET;
                        break;

                    default:
                        break;
                }

                this.IsBasicAuth = IsBasicAuth;
                this.BasicAuthUser = BasicAuthUser;
                this.BasicAuthPassword = BasicAuthPassword;
                this.Parameters = Parameters;
                this.PostData = PostData;
                this.Headers = Headers;
            }

            /// <summary>
            /// url
            /// </summary>
            public string Url { get; set; } = string.Empty;

            /// <summary>
            /// 请求方法
            /// </summary>
            public RequestMethodType RequestMethod { get; set; } = RequestMethodType.GET;

            /// <summary>
            /// 是否有权限验证
            /// </summary>
            public bool IsBasicAuth { get; set; } = false;

            /// <summary>
            /// 权限账户
            /// </summary>
            public string BasicAuthUser { get; set; } = string.Empty;

            /// <summary>
            /// 账户密码
            /// </summary>
            public string BasicAuthPassword { get; set; } = string.Empty;

            /// <summary>
            /// 请求参数
            /// </summary>
            public IDictionary<string, string> Parameters { get; set; } = new Dictionary<string, string>();

            /// <summary>
            /// Post请求数据
            /// </summary>
            public string PostData { get; set; } = string.Empty;

            /// <summary>
            /// 请求头
            /// </summary>
            public IDictionary<string, string> Headers { get; set; } = new Dictionary<string, string>();

            /// <summary>
            /// 编码格式
            /// </summary>
            public Encoding dataEncoding { get; set; } = Encoding.UTF8;

            /// <summary>
            /// 默认参数格式
            /// </summary>
            public string ContentType { get; set; } = "Content-Type:application/json; charset=utf-8";
        }
}