﻿using System;
using System.Collections.Generic;
using System.Text;

namespace OrBit.MESInterface.Infrastructure.Common
{
    /// <summary>
    /// 雪花ID
    /// Twitter_Snowflake
    /// SnowFlake的结构如下(每部分用-分开)
    /// 0 - 0000000000 0000000000 0000000000 0000000000 0 - 00000 - 00000 - 000000000000
    /// 1位标识，由于long基本类型在Java中是带符号的，最高位是符号位，正数是0，负数是1，所以id一般是正数，最高位是0
    /// 41位时间截(毫秒级)，注意，41位时间截不是存储当前时间的时间截，而是存储时间截的差值（当前时间截 - 开始时间截)得到的值），
    /// 41位的时间截，可以使用69年，年T = (1L << 41) / (1000L * 60 * 60 * 24 * 365) = 69
    /// 这里的的开始时间截，一般是我们的id生成器开始使用的时间，由我们程序来指定的（如下下面程序IdWorker类的startTime属性）。
    /// 10位的数据机器位，可以部署在1024个节点，包括5位datacenterId和5位workerId
    /// 12位序列，毫秒内的计数，12位的计数顺序号支持每个节点每毫秒(同一机器，同一时间截)产生4096个ID序号
    /// 总共加起来刚好64位，为一个Long型。
    /// SnowFlake的优点是，整体上按照时间自增排序，并且整个分布式系统内不会产生ID碰撞(由数据中心ID和机器ID作区分)，
    /// 并且效率较高，经测试，SnowFlake单机每秒都能够产生出极限4,096,000个ID来
    /// </summary>
    public class SnowflakeTool
    {
        Snowflake snowflake = null;


        Stack<long> datacenterIdStack = new Stack<long>(new List<long> { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31 });
        Stack<long> workerIdStack = new Stack<long>(new List<long> { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31 });
        public SnowflakeTool()
        {         

          var  datacenterId = //new Random().Next(1, 31);//
                                                      datacenterIdStack.Pop();
          var  workerId = //new Random().Next(1, 31);//
                                                workerIdStack.Pop();
            snowflake = new Snowflake(datacenterId, workerId);
        }


        /// <summary>
        /// 获取ID
        /// </summary>
        /// <returns></returns>
        public long NextId()
        {
            return snowflake.NextId();
        }

        /// <summary>
        /// 解析ID
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public string AnalyzeId(long id)
        {
            return Snowflake.AnalyzeId(id);
        }
    }
}
