﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using OrBit.MESInterface.Infrastructure.SQLTemplate;
using System;
using System.Collections.Generic;
using System.Linq;

namespace OrBit.MESInterface.Infrastructure.CreateTable
{
    public class CreateTableHelper
    {
        public static string CheckTableExists = string.Empty;

        public static string QueryDBTableInfo = string.Empty;

        /// <summary>
        /// 对比数据库表字段是否一致
        /// </summary>
        /// <param name="tableModel"></param>
        /// <param name="dBTable"></param>
        /// <returns></returns>
        public static void TableDiff(TableModel tableModel, List<DBTableInfo> dBTable)
        {
            tableModel.ColumnDescription.Keys.ToList().ForEach(col =>
            {
                if (!dBTable.Any(dt => dt.ColumnName == col))
                {
                    tableModel.AlterColumnDescription.Add(col, tableModel.ColumnDescription[col]);
                }
            });

            //    return tableModel;
        }

        /// <summary>
        /// 构建新表model
        /// </summary>
        /// <param name="TableName">表名</param>
        /// <param name="jlist">要保存的表列 json数组格式</param>
        /// <returns></returns>
        public static TableModel BuildTableModel(string TableName, JArray jlist, string id = "")
        {
            TableModel tableModel = new TableModel(TableName, id);

            tableModel.Data.Columns.Add(tableModel.GroupIDName, Type.GetType("System.String"));
            tableModel.ColumnDescription.Add(tableModel.GroupIDName, "String");
            tableModel.Data.Columns.Add(tableModel.GroupCreateTimeName, Type.GetType("System.DateTime"));
            tableModel.ColumnDescription.Add(tableModel.GroupCreateTimeName, "DateTime");
            QueryDBTableInfo = CreateTableSQLTemplate.DBTableInfo.Replace("{tableName}", tableModel.TableName);

            CheckTableExists = CreateTableSQLTemplate.TableIsExists.Replace("{tableName}", tableModel.TableName);
            if (jlist.Count > 0)
            {
                //取第一个创建表
                var Firstreader = jlist.First.CreateReader();
                while (Firstreader.Read())
                {
                    if (Firstreader.Value != null)
                    {
                        if (Firstreader.TokenType == JsonToken.PropertyName)
                        {
                            var PropertyName = Firstreader.Value?.ToString();
                            Firstreader.Read();
                            if (!string.IsNullOrWhiteSpace(PropertyName))
                            {
                                tableModel.ColumnDescription.Add(PropertyName, Firstreader.ValueType.Name);
                                //  tableModel.Data.Columns.Add(PropertyName, Type.GetType(Firstreader.ValueType.FullName));
                                tableModel.Data.Columns.Add(PropertyName, Type.GetType("System.String"));
                            }
                        }
                    }
                }
                for (int i = 0; i < jlist.Count; i++)//循环所有填充数据
                {
                    var reader = jlist[i].CreateReader();
                    var row = tableModel.Data.NewRow();
                    row[tableModel.GroupIDName] = tableModel.GroupIDValue;
                    row[tableModel.GroupCreateTimeName] = tableModel.GroupCreateTimeValue;
                    while (reader.Read())
                    {
                        if (reader.Value != null)
                        {
                            var PropertyName = reader.Value?.ToString();
                            reader.Read();
                            // row[PropertyName].GetType()
                            // Console.WriteLine($"Column：{PropertyName}，Type{ tableModel.Data.Columns[PropertyName].DataType.Name},value：{reader.Value}");

                            row[PropertyName] = Convert.ChangeType(reader.Value, tableModel.Data.Columns[PropertyName].DataType);
                        }
                    }
                    tableModel.Data.Rows.Add(row);
                }
            }
            tableModel.Data.TableName = tableModel.TableName;
            return tableModel;
        }

        public static string BulidCreateTableSQL(TableModel tableModel)
        {
            IBuildTableSqlService buildTableSqlService = new BuildTableSqlService();
            buildTableSqlService.SettingTableName(tableModel.TableName, tableModel.PrimaryKey)
                                   .AddColumn(tableModel.GroupIDName, "nvarchar(36)")
                                   .AddColumn(tableModel.GroupCreateTimeName, "datetime");

            foreach (var key in tableModel.ColumnDescription.Keys)
            {
                if (key == tableModel.GroupIDName || key == tableModel.GroupCreateTimeName)
                    continue;
                string type = "";
                switch (tableModel.ColumnDescription[key].ToLower())
                {
                    case "int64":
                        type = "nvarchar(100)";
                        break;

                    case "string":
                        type = "nvarchar(max)";
                        break;

                    case "double":
                    case "float":
                        type = "nvarchar(100)";
                        break;

                    case "bool":
                        type = "bit";
                        break;

                    default:
                        type = "nvarchar(max)";

                        break;
                }
                buildTableSqlService.AddColumn(key, type);
            }
            return buildTableSqlService.SettingDefaultValue(tableModel.PrimaryKey, "newid()")
                                   .BullSQL();
        }

        public static string BulidNewFieldSQL(TableModel tableModel)
        {
            IBuildTableSqlService buildTableSqlService = new BuildTableSqlService();

            foreach (var key in tableModel.AlterColumnDescription.Keys)
            {
                if (key == tableModel.GroupIDName || key == tableModel.GroupCreateTimeName)
                    continue;
                string type = "";
                switch (tableModel.ColumnDescription[key].ToLower())
                {
                    case "int64":
                        type = "nvarchar(100)";
                        break;

                    case "string":
                        type = "nvarchar(max)";
                        break;

                    case "double":
                    case "float":
                        type = "nvarchar(100)";
                        break;

                    case "bool":
                        type = "bit";
                        break;

                    default:
                        type = "nvarchar(max)";

                        break;
                }
                buildTableSqlService.NewFieldColumn(tableModel.TableName, key, type);
            }
            return buildTableSqlService.BullSQL();
        }
    }
}