﻿using Newtonsoft.Json;
using OrBit.MESInterface.Infrastructure.Common;
using OrBit.MESInterface.Infrastructure.DataSetToJSON;
using OrBit.MESInterface.Infrastructure.Extensions;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OrBit.MESInterface.Infrastructure.DataSetToJson
{
    public class DefaultDataSet2JsonService : IDataSet2JsonService
    {
        public List<Dictionary<string, object>> BuildDictionary(DataSet ds)
        {
            List<TemplateDataSourceInfo> dataSourceInfos = new List<TemplateDataSourceInfo>();

            foreach (DataTable item in ds.Tables)
            {
                if (!item.Columns.Contains("TableName") || !item.Columns.Contains("TableInfo") || !item.Columns.Contains("ParentColumnName") || !item.Columns.Contains("ParentColumnValue"))
                {
                    throw new Exception($"请求参数数据集第{ds.Tables.IndexOf(item) + 1}张表必须包涵“TableName”，“TableInfo”，“ParentColumnName”，“ParentColumnValue”4个固定字段。请检查！");
                }

                var Tables = item.AsEnumerable().Select(row => new TemplateDataSourceInfo
                {
                    TableName = row["TableName"].ToString(),
                    TableInfo = row["TableInfo"].ToString().Equals("root") ? "root" : "",
                    ParentColumnName = row["ParentColumnName"].ToString(),
                    ParentColumnValue = row["ParentColumnValue"],
                }).DistinctPro(tb => tb.TableName + tb.ParentColumnValue).ToList();
                //给每一行分配数据源
                Tables.ForEach(tb =>
                {
                    var tmpdt = item.Copy();
                    var rows = item.Rows.Cast<DataRow>().Where(row => row["TableName"].Equals(tb.TableName) && (string.IsNullOrEmpty(tb.ParentColumnName) || row["ParentColumnValue"].Equals(tb.ParentColumnValue))).ToList(); //给每一行分配数据源
                    tmpdt.Columns.Remove("TableName");
                    tmpdt.Columns.Remove("TableInfo");
                    tmpdt.Columns.Remove("ParentColumnName");
                    tmpdt.Columns.Remove("ParentColumnValue");

                    DataTable newDT = tmpdt.Clone();
                    rows.ForEach(row =>
                    {
                       
                        newDT.ImportRow(row);
                    });

                    for (int i = 0; i < newDT.Columns.Count; i++)
                    {
                        var columnName = tmpdt.Columns[i].ColumnName;
                        if (columnName.Contains(" as "))
                        {
                            newDT.Columns[i].ColumnName = columnName.Replace(columnName.Substring(0, columnName.LastIndexOf(' ')+1), "");
                        }
                    }

                    //tb.DataSourceDT = newDT;
                    tb.DataSourceList = newDT.ToList();
                });
                dataSourceInfos.AddRange(Tables);
            }
            var haveRootCount = dataSourceInfos.Where(tds => tds.TableInfo.Equals("root")).Count();
            if (haveRootCount != 1)
                throw new Exception($"请求参数数据集未包涵root信息的表。请保证数据集有且只有一个root表。“root”填写在“TableInfo”字段中。");

            var rootDataSource = dataSourceInfos.SingleOrDefault(d => d.TableInfo == "root");

            BuildFillParameterTemplateDataSource(rootDataSource, dataSourceInfos);
            return rootDataSource.DataSourceList;
        }

        public string BuildJSON(DataSet ds, string jsonType)
        {
            var dic = BuildDictionary(ds);

            if (jsonType.Equals("[]"))
                return JsonConvert.SerializeObject(dic, Formatting.Indented);
            else
                return JsonConvert.SerializeObject(dic.FirstOrDefault(), Formatting.Indented);
        }

        /// <summary>
        /// 使用递归函数从第一次解析到最后一层。
        /// </summary>
        /// <param name="dataSourceInfo">要解析的数据源</param>
        /// <param name="dataSources">填充到上层的数据源</param>
        /// <param name="dataType">数据类型</param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        private object BuildFillParameterTemplateDataSource(TemplateDataSourceInfo dataSourceInfo, List<TemplateDataSourceInfo> dataSources, string dataType = "", bool zip = false)
        {
            var dtList = dataSourceInfo.DataSourceList;
            var FillColumnList = dtList[0].Where(d => d.Key.Contains(":fill")).Select(d => d.Key).ToList();
            FillColumnList.ForEach(key =>
            {
                var column = key;
                var gzip = false;
               
                column = column.Replace(":fill", "");

                dtList.ForEach(d =>
                {
                    var value = d[key];
                    d.Remove(key);
                    d.Add(column, value);

                    var ColumnValue = d[column].ToString();

                    if (!ColumnValue.Contains(':'))
                    {
                        throw new Exception($"获取请求参数时，表{dataSourceInfo.TableName},对应的填充字段{key}关键字{ColumnValue}错误。请参照示例检查！示例：数据源表名:{{}}");
                    }

                    var tableName = ColumnValue.Substring(0, ColumnValue.IndexOf(":"));
                    var dataType = ColumnValue.Replace($"{tableName}:", "").Replace($":gzip", "");
                    if (ColumnValue.Contains("gzip"))
                        gzip = true;
                    var fillDataSources = dataSources.Where(ds => ds.TableName.Equals(tableName) && (string.IsNullOrWhiteSpace(ds.ParentColumnName) || (d.Keys.Contains(ds.ParentColumnName) && d[ds.ParentColumnName].Equals(ds.ParentColumnValue)))).ToList();
                    if (fillDataSources.Count <= 0)
                    {
                        if (!dataType.Contains("null"))
                            throw new Exception($"获取请求参数时，表{dataSourceInfo.TableName},对应的填充字段{key}的数据源{tableName}未获取到数据。请检查。如果不需要填充请修改表字段不要包含\":fill\"。如果当前数据集可为空请在表字段{dataType}上添加\":null\"关键字。示例:{dataType}:null");
                        else
                        {
                            d[column] = ConfigFillData(new List<Dictionary<string, object>> { }, dataType, gzip);
                        }
                    }

                    fillDataSources.ForEach(fds =>
                    {
                        if (fds != null && fds.DataSourceList != null && fds.DataSourceList.Count > 0)
                        {
                            d[column] = BuildFillParameterTemplateDataSource(fds, dataSources, dataType, gzip);
                        }
                    });
                });
            });
            dtList.ForEach(dl =>
            {
                var removeKey = dl.Keys.Where(key => key.ToUpper().EndsWith("__KEYID__")).ToList();
                dl.RemoveAll(d => removeKey.Contains(d.Key));
            });
            return ConfigFillData(dtList, dataType,zip);
        }

        private object ConfigFillData(List<Dictionary<string, object>> DataSource, string DataType,bool zip)
        {
            object filldata;

            switch (DataType)
            {
                case "{}":
                case "{}:null":
                    filldata = DataSource.FirstOrDefault();
                    break;

                case "":
                case "[{}]":
                case "[{}]:null":
                    filldata = DataSource;
                    break;

                default:
                    filldata = JsonHelper.ToStringList(DataSource);
                    break;
            }
            if(zip)
            {
               return GZipUtillib.GZipCompressString(JsonConvert.SerializeObject(filldata));             
            }
            return filldata;
        }
    }
}