﻿using Newtonsoft.Json;

namespace OrBit.MESInterface.Infrastructure
{
    public enum ExecutionState
    {
        /// <summary>
        /// 返回错误信息
        /// </summary>
        FAIL = -1,

        /// <summary>
        /// 请求成功，但是没有新数据
        /// </summary>
        OK = 0,

        /// <summary>
        /// 请求成功，返回新数据
        /// </summary>
        SUCCESS = 1,

        /// <summary>
        /// 数据保存成功
        /// </summary>
        SaveDataSuccess = 2,

        /// <summary>
        /// 数据保存业务表成功
        /// </summary>
        SaveTooBusinessTableSuccess = 3
    }

    public class ExecutionResult
    {
        /// <summary>
        /// 解析状态 -1：返回错误信息;0:请求成功，但是没有新数据;1：请求成功，返回新数据;2:数据保存成功; 3：数据保存业务表成功
        /// </summary>

        public ExecutionResult()
        {
            code = ExecutionState.FAIL;
        }

        public ExecutionResult(ExecutionState _State, string _Message, object _Value = null, dynamic _Result = null)
        {
            code = _State;
            message = _Message;
            details = _Value;
            data = _Result;
        }

        public static ExecutionResult Success(string msg = "", object data = null, object details = null)
        {
            return new ExecutionResult(ExecutionState.SUCCESS, msg, details, data);
        }

        public static ExecutionResult Failed(string msg = "", object data = null)
        {
            return new ExecutionResult(ExecutionState.FAIL, msg, data);
        }

        public string ToJson()
        {
            return JsonConvert.SerializeObject(this);
        }

        public string DataToJson()
        {
            return JsonConvert.SerializeObject(this.data);
        }

        public ExecutionState code { get; set; }

        public string message { get; set; }

        public object data { get; set; }

        public object details { get; set; }
        public string validationErrors { get; set; }
    }
}