﻿using Azure.Core;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Options;
using System.Threading.Tasks;

namespace OrBit.MESInterface.Infrastructure.Explorer
{
    public class ExplorerMiddleware : IMiddleware
    {
        private readonly IExplorer explorer;
        public ExplorerMiddleware(IExplorer explorer) { 
            this.explorer = explorer;
        }
        public async Task InvokeAsync(HttpContext context, RequestDelegate next)
        {
            if(context.Request.Path.StartsWithSegments("/Explorer/QueryDrive"))
            {                
                context.Response.StatusCode = 200;
                await context.Response.WriteAsync(ExecutionResult.Success(data: explorer.GetDriveItem()).ToJson());
                return;
            }
            if (context.Request.Path.StartsWithSegments("/Explorer/QueryDirectoryAndFile"))
            {
                if(!context.Request.Query.ContainsKey("directory"))
                {
                    context.Response.StatusCode = 400;
                    await context.Response.WriteAsync(ExecutionResult.Success("未指定路径").ToJson());
                    return;
                }
                var directory = context.Request.Query["directory"];
                context.Response.StatusCode = 200;
                await context.Response.WriteAsync(ExecutionResult.Success(data: explorer.GetDirectoryAndFile(directory)).ToJson());
                return;
            }

            if (context.Request.Path.StartsWithSegments("/Explorer/UploadFile"))
            {
                if(context.Request.Method != "POST")
                {
                    context.Response.StatusCode = 400;
                    await context.Response.WriteAsync(ExecutionResult.Success("请使用POST方法").ToJson());
                    return;
                }
                var contentType = context.Request.Headers["Content-Type"].ToString();
                if (!contentType.Contains("multipart/form-data"))
                {
                    context.Response.StatusCode = 400;
                     await context.Response.WriteAsync(ExecutionResult.Failed("ContentType内容必须是multipart/form-data").ToJson());
                    return;

                }

                if (context.Request.Form.Files.Count <= 0)
                {
                    context.Response.StatusCode = 400;
                    await context.Response.WriteAsync(ExecutionResult.Failed("请添加文件").ToJson());
                    return;
                }

                await context.Response.WriteAsync(ExecutionResult.Success(data: explorer.GetDirectoryAndFile("")).ToJson());
                return;
            }
        }
    }
}
