﻿using OrBit.MESInterface.Infrastructure.Common;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static OrBit.MESInterface.Infrastructure.Explorer.Models.CustomFileInfo;

namespace OrBit.MESInterface.Infrastructure.Explorer.Models
{
    public abstract class ExplorerItem
    {
        public ExplorerItem(string fullName)
        {
            Image = GetImage(fullName);
            Name = GetDirectoryName(fullName);
            FullName = fullName;
            DisplayName = GetDisplayName(fullName);
        }

        public virtual string ImageStr { get { return ConvertObjectToBase64.BitmapToBase64(Image); } }
        protected virtual string GetDisplayName(string fullName)
        {
            return Name;
        }
        public string DisplayName
        {
            get;
            private set;
        }
        public string Name
        {
            get;
            private set;
        }
        public string FullName
        {
            get;
            private set;
        }
        public virtual Image Image
        {
            get;
            set;
        }
        public abstract List<ExplorerItem> GetDirectories();
        public static Size ImageSize
        {
            get;
            set;
        }
        protected Image GetImage(string fullName,bool drive=false)
        {
            if (drive)
                return SystemIcon.GetDriver(fullName);

            return SystemIcon.GetImage(fullName, drive );
        }
        protected virtual string GetDirectoryName(string fullName)
        {
            return fullName;
        }
        public virtual List<CustomFileInfo> GetFilesSystemInfo()
        {
            List<CustomFileInfo> infos = new List<CustomFileInfo>(15);
            try
            {
                DirectoryInfo dInfo = new DirectoryInfo(FullName);
                if ((int)dInfo.Attributes == -1)
                    return infos;
                var directories = dInfo.GetDirectories();
                foreach (var directory in directories)
                {
                    if ((directory.Attributes & FileAttributes.Hidden) != FileAttributes.Hidden)
                    {
                        infos.Add(new CustomFileInfo()
                        {
                            Name = directory.Name,
                            FullName = directory.FullName,
                            DateCreated = directory.CreationTime,
                            DateModified = directory.LastWriteTime,
                            Image = GetImage(directory.FullName),
                            TypeName = "File Folder",
                            Type = FileType.FileFolder
                        });
                    }
                }
                var files = dInfo.GetFiles();
                foreach (var file in files)
                {
                    if ((file.Attributes & FileAttributes.Hidden) != FileAttributes.Hidden)
                    {
                        infos.Add(new CustomFileInfo()
                        {
                            Name = file.Name,
                            FullName = file.FullName,
                            DateCreated = file.CreationTime,
                            DateModified = file.LastWriteTime,
                            Image = GetImage(file.FullName),
                            TypeName = "File",
                            Type = FileType.File,
                            Size = file.Length
                        });
                    }
                }
            }
            catch { }
            return infos;
        }
    }
}
