﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace OrBit.MESInterface.Infrastructure.Extensions
{
    public class Compare<T, C> : IEqualityComparer<T>
    {
        private Func<T, C> _getField;

        public Compare(Func<T, C> getfield)
        {
            this._getField = getfield;
        }

        public bool Equals(T x, T y)
        {
            return EqualityComparer<C>.Default.Equals(_getField(x), _getField(y));
        }

        public int GetHashCode(T obj)
        {
            return EqualityComparer<C>.Default.GetHashCode(this._getField(obj));
        }
    }

    public static class CommonHelper
    {
        /// <summary>
        /// 自定义Distinct扩展方法
        /// </summary>
        /// <typeparam name="T">要去重的对象类</typeparam>
        /// <typeparam name="C">自定义去重的字段类型</typeparam>
        /// <param name="source">要去重的对象</param>
        /// <param name="getfield">获取自定义去重字段的委托</param>
        /// <returns></returns>
        public static IEnumerable<T> DistinctPro<T, C>(this IEnumerable<T> source, Func<T, C> getfield)
        {
            return source.Distinct(new Compare<T, C>(getfield));
        }

        public static T ObjCast<T>(this object obj, T sample)
        {
            return (T)obj;
        }

        public static List<T> ConvertToDesiredType<T>(this object list) where T : new()
        {
            List<T> returnList = new List<T>();

            foreach (var item in (IEnumerable<dynamic>)list)
            {
                T obj = JsonConvert.DeserializeObject<T>(item.ToString().Replace("{{", "{").Replace("}}", "}"));
                T extract = new T();
                foreach (PropertyInfo prop in obj.GetType().GetProperties())
                {
                    foreach (PropertyInfo prop2 in extract.GetType().GetProperties())
                    {
                        if (prop2.Name == prop.Name)
                        {
                            prop2.SetValue(extract, prop.GetValue(obj));
                        }
                    }
                }

                returnList.Add(extract);
            }

            return returnList;
        }
    }
}