﻿using Microsoft.IdentityModel.Tokens;
using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;

namespace OrBit.MESInterface.Infrastructure.Jwt
{
    public class JwtTool
    {
        /// <summary>
        /// JWT配置
        /// </summary>
        private JwtBearerSetting _jwtBearerSetting;

        public JwtTool()
        {
        }

        public JwtTool(JwtBearerSetting jwtBearerSetting)
        {
            _jwtBearerSetting = jwtBearerSetting;
        }

        public void ConfigJwtParameters(JwtBearerSetting jwtBearerSetting)
        {
            _jwtBearerSetting = jwtBearerSetting;
        }

        /// <summary>
        /// 生成AccessToken
        /// </summary>
        /// <param name="username">这里测试用的是用户信息，可以传入其他信息</param>
        /// <returns></returns>
        public string GenerateAccessToken(string username)
        {
            if (_jwtBearerSetting == null)
                throw new Exception("未初始化jwt配置参数");

            var claims = new[]
            {
             new Claim(JwtRegisteredClaimNames.Sub,username),
            // new Claim(ClaimTypes.Role,"admin")
            new Claim(JwtRegisteredClaimNames.Iat,DateTimeOffset.Now.ToUnixTimeSeconds().ToString())
         };
            //生成token
            var key = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(_jwtBearerSetting.SecurityKey));
            var creds = new SigningCredentials(key, SecurityAlgorithms.HmacSha256);

            var securityToken = new JwtSecurityToken(
                issuer: _jwtBearerSetting.Issuer,   // 发布者
                notBefore: DateTime.Now,            // token签发时间
                audience: _jwtBearerSetting.Audience,  // 接收者
                claims: claims,                         // 该token内存储的自定义字段信息
                expires: DateTime.Now.AddDays(1),             // token过期时间

                signingCredentials: creds);              // 用于签发token的秘钥算法

            var token = new JwtSecurityTokenHandler().WriteToken(securityToken);
            return token;
        }

        public string CustomizationGenerateAccessToken(string username)
        {
            if (_jwtBearerSetting == null)
                throw new Exception("未初始化jwt配置参数");

            var claims = new[]
            {
             new Claim(JwtRegisteredClaimNames.Sub,username),
            // new Claim(ClaimTypes.Role,"admin")
            new Claim(JwtRegisteredClaimNames.Iat,DateTimeOffset.Now.ToUnixTimeSeconds().ToString())
         };
            //生成token
            var key = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(_jwtBearerSetting.SecurityKey));
            var creds = new SigningCredentials(key, SecurityAlgorithms.HmacSha256);

            var securityToken = new JwtSecurityToken(
                issuer: _jwtBearerSetting.Issuer,   // 发布者
                                                    //  notBefore: DateTime.Now,            // token签发时间
                                                    //audience: _jwtBearerSetting.Audience,  // 接收者
                claims: claims,                         // 该token内存储的自定义字段信息
                                                        //expires: DateTime.Now.AddDays(1),             // token过期时间

                signingCredentials: creds);              // 用于签发token的秘钥算法

            var token = new JwtSecurityTokenHandler().WriteToken(securityToken);
            return token;
        }
    }
}