using Microsoft.Data.SqlClient;
using OrBit.MESInterface.OrderVerifyWebService.Domain.Entities;
using OrBit.MESInterface.OrderVerifyWebService.Domain.InterfaceConfiguration.Repository;
using OrBit.MESInterface.OrderVerifyWebService.EntiyFrameworkCore.Extensions;
using OrBit.MESInterface.OrderVerifyWebService.EntiyFrameworkCore;


using Volo.Abp;
using Volo.Abp.Domain.Repositories.EntityFrameworkCore;
using Volo.Abp.EntityFrameworkCore;

using OrBit.MESInterface.OrderVerifyWebService.WebApi.Infrastructure.Common;

namespace OrBit.MESInterface.EntiyFrameworkCore.Repositories;

public class InterfaceConfigurationRepository : EfCoreRepository<MesInterfaceDbContext, MESInterfaceConfiguration>,
    IInterfaceConfigurationRepository
{

    
    public InterfaceConfigurationRepository(IDbContextProvider<MesInterfaceDbContext> _dbContextProvider) : base(
        _dbContextProvider)
    {
        
    }

    //public  async Task<IEnumerable<MESInterfaceConfiguration>> IncludeDetails(string interfaceServiceName, bool include,
    //    CancellationToken cancellationToke = default)
    //{
    //    await DbContext.MesInterfaceConfigurations.Include(include,Configur=>Configur.InterfaceServiceCode)
    //}

    public override async Task<IQueryable<MESInterfaceConfiguration>> WithDetailsAsync()
    {
        return (await GetQueryableAsync()).IncludeDetails();
    }

    public async Task<string> FillParameterTemplate(InterfaceServiceCode serviceCode, DateTime QueryDateEnd,
        string QueryParameters)
    {
        var sql =
            $@"if exists(select * from dbo.sysobjects where id = object_id(N'[dbo].[{serviceCode.FillParameterTemplateByStoredProcedure}]') and OBJECTPROPERTY(id, N'IsProcedure') = 1) begin
	                       select 'exists' as ResultMSG
                            end
                            else
                             select 'no exists' as ResultMSG";
        var dbContext = await GetDbContextAsync();
        var ds = await dbContext.SqlQueryResultDataSetAsync(sql);
        if (ds != null && ds.Tables.Count > 0 && ds.Tables[ds.Tables.Count - 1].Rows.Count > 0 &&
            ds.Tables[ds.Tables.Count - 1].Rows[0][0].ToString().ToUpper().Equals("no exists"))
        {
            throw new UserFriendlyException(
                $"当前接口服务类型:{serviceCode.InterfaceServiceName}，获取请求参数的存储过程[{serviceCode.FillParameterTemplateByStoredProcedure}]不存在！",
                "-1", logLevel: LogLevel.Error);
        }

        var result = await dbContext.ExecuteStoredProcedureResultDataSetAsync(
            serviceCode.FillParameterTemplateByStoredProcedure,
            new SqlParameter[]
            {
                new SqlParameter("@Interface_Request_ConfigurationID", serviceCode.MESInterfaceConfigurationID),
                new SqlParameter("@InterfaceServiceName", serviceCode.InterfaceServiceName),
                new SqlParameter("@QueryDateEnd", QueryDateEnd.ToString("yyyy-MM-dd HH:mm:ss")),
                new SqlParameter("@QueryParameters", QueryParameters)
            });

        if (result == null || result.Tables.Count <= 0 || result.Tables[0].Rows.Count <= 0)
            throw new UserFriendlyException($"当前接口服务类型:{serviceCode.InterfaceServiceName}，没有获取到请求参数", "-1",
                logLevel: LogLevel.Error);

        if (result.Tables.Count > 1)
        {
            int tableCount = result.Tables.Count - 1;
            //for ( tableCount > 0; tableCount--)
            //{
            for (int i = result.Tables[0].Columns.Count - 1; i > 0; i--)
            {
                if (result.Tables[0].Rows[0][i].ToString() == "fill")
                {
                    var tmpjson = JsonHelper.Dtb2Json(result.Tables[tableCount]);
                    result.Tables[0].Rows[0][i] = tmpjson;
                    tableCount--;
                }
            }
        }

        if (result.Tables.Count > 0 && result.Tables[0].Rows.Count > 0)
        {
            return JsonHelper.JsonTemplateReplacement_Pro(serviceCode.ParameterTemplate, result.Tables[0]);
        }
        else
            return null;
    }
}