﻿using System.Text.RegularExpressions;
using System.Text;
using Volo.Abp;
using OrBit.MESInterface.OrderVerifyWebService.Application;

namespace OrBit.MESInterface.OrderVerifyWebService.Middleware
{
    public class BasicauthCheckMiddleware
    {
        private readonly RequestDelegate _next;

        private readonly ILogger<BasicauthCheckMiddleware> logger;

        public BasicauthCheckMiddleware(RequestDelegate next, ILogger<BasicauthCheckMiddleware> _logger)
        {
            _next = next;
            logger = _logger;
        }

        public async Task InvokeAsync(HttpContext context)
        {
            if (context.Request.Path.Value.Equals("/Mes/MesService.asmx"))
            {
                AnalyticResult result = new AnalyticResult();
                string authHeader = context.Request.Headers["Authorization"];
                var ip = context.Connection.RemoteIpAddress;//Current.Request.UserHostAddress;

                if (authHeader != null && authHeader.StartsWith("Basic"))
                {
                    //Extract credentials
                    string encodedUsernamePassword = authHeader.Substring("Basic ".Length).Trim();
                    Encoding encoding = Encoding.GetEncoding("iso-8859-1");
                    string usernamePassword = encoding.GetString(Convert.FromBase64String(encodedUsernamePassword));
                    int seperatorIndex = usernamePassword.IndexOf(':');
                    var username = usernamePassword.Substring(0, seperatorIndex);
                    var password = usernamePassword.Substring(seperatorIndex + 1);

                    if (!Regex.IsMatch(username, @"^[A-Za-z0-9_]+$"))
                    {
                        logger.LogInformation("Wrong user name format");
                        result.code = AnalyticState.FAIL;
                        result.message = "Wrong user name format";
                        await context.Response.WriteAsJsonAsync(result, typeof(AnalyticResult));
                        return;
                    }

                    //if (IsAuthorized(username, password))
                    if (ValidateUser(username, password))
                    {
                        logger.LogInformation($"Authorized, user {username}");

                        return;
                    }
                    else
                    {
                        logger.LogInformation($"Unauthorized, user {username} ");
                        result.code = AnalyticState.FAIL;
                        result.message = "用户名密码错误";
                        await context.Response.WriteAsJsonAsync(result, typeof(AnalyticResult));
                        return;
                    }
                }
                else
                {
                    logger.LogInformation($"No Basic Auth Info ");
                    result.code = AnalyticState.FAIL;
                    result.message = "No Basic Auth Info";
                    await context.Response.WriteAsJsonAsync(result, typeof(AnalyticResult));
                    return;
                }
            }
            await _next(context);
        }

        public bool ValidateUser(string accountName, string password)
        {
            //验证用户名密码

            return "sapuser".Equals(accountName, StringComparison.InvariantCultureIgnoreCase)
                   && "12345".Equals(password); ;
        }
    }
}