using System.Collections.ObjectModel;
using System.Data;
using Serilog;
using Serilog.Core;
using Serilog.Events;
using Serilog.Sinks.MSSqlServer;

namespace OrBit.MESInterface.OrderVerifyWebService;

public class SerilogConfiguration
{
    private readonly IConfiguration configuration;

    public SerilogConfiguration(IConfiguration configuration)
    {
        this.configuration = configuration;
    }

    public Logger CreateLogger()
    {
        const string logformat = @"{Timestamp:yyyy-MM-dd HH:mm:ss }[{Level:u3}] {Message:lj}{NewLine}{UserName}";
        var connectionString = configuration.GetSection("Serilog:MSSqlServer:ConnectionString").Value;
        var columnOptions = ConfigurationColumnOts();
        var msSqlServerSinkOptions = ConfigurationMSSqlServerSinkOptions();
        var logger = new LoggerConfiguration()
            //.MinimumLevel.Override("Microsoft", Serilog.Events.LogEventLevel.Error)
            .MinimumLevel.Override("Microsoft.Hosting.Lifetime", LogEventLevel.Error)
            .MinimumLevel.Override("Volo", Serilog.Events.LogEventLevel.Error)
            .Enrich.FromLogContext()
            .WriteTo.Async(c => c.Console())
            .WriteTo.Async(c => c.File("Logs/Log.txt", LogEventLevel.Information, rollingInterval: RollingInterval.Day, outputTemplate: logformat))
            .WriteTo.Async(c => c.MSSqlServer(connectionString,
                sinkOptions: msSqlServerSinkOptions, columnOptions: columnOptions
                , restrictedToMinimumLevel: LogEventLevel.Information))
            .CreateLogger();

        return logger;
    }

    public ColumnOptions ConfigurationColumnOts()
    {
        return new ColumnOptions
        {
            AdditionalColumns = new Collection<SqlColumn>
            {
                new SqlColumn
                    {ColumnName = "InterfaceType", PropertyName = "InterfaceType", DataType = SqlDbType.NVarChar, DataLength = 100},
                new SqlColumn
                    {ColumnName = "InterfaceServiceName", DataType=SqlDbType.NVarChar, DataLength = 100},
                new SqlColumn
                    {ColumnName = "ActionId", DataType = SqlDbType.NVarChar, DataLength = -1, AllowNull = true},
                new SqlColumn
                    {ColumnName = "ActionName", DataType = SqlDbType.NVarChar, DataLength = -1, AllowNull = true},
                new SqlColumn
                    {ColumnName = "QueryString", DataType = SqlDbType.NVarChar, DataLength = -1, AllowNull = true},
            }
        };
    }

    public MSSqlServerSinkOptions ConfigurationMSSqlServerSinkOptions()
    {
        var mssqloption = new MSSqlServerSinkOptions();
        mssqloption.TableName = configuration.GetSection("Serilog:MSSqlServer:TableName").Value;
        mssqloption.BatchPostingLimit = configuration.GetValue<int>("Serilog:MSSqlServer:BatchPostingLimit");
        mssqloption.AutoCreateSqlTable = true;

        return mssqloption;
    }
}